Package: paneltests
Type: Package
Title: Panel Data Pre-Testing and Diagnostic Suite
Version: 1.0.5
Authors@R: person("Muhammad Abdullah", "Alkhalaf", email = "muhammedalkhalaf@gmail.com",
    role = c("aut", "cre", "cph"),
    comment = c(ORCID = "0009-0002-2677-9246"))
Description: Pre-testing and diagnostic tools for panel data analysis.
    Researchers should run these tests before any panel regression to
    verify modelling assumptions. The package implements: (1) the Hsiao
    (2014, <ISBN:978-1-107-65763-2>) homogeneity F-tests (F1/F2/F3),
    Swamy (1970) <doi:10.2307/1913012> parameter heterogeneity test, and
    Pesaran (2004) <doi:10.2139/ssrn.572504> cross-sectional dependence
    test via xtpretest(); (2) missing-data detection, mechanism testing,
    and imputation for unbalanced panels via xtmispanel(); (3)
    quantile-regression cross-sectional dependence tests (T_tau and
    T-tilde_tau statistics) of Demetrescu, Hosseinkouchack and Rodrigues
    (2023) <doi:10.1016/j.jeconom.2022.09.001> via xtcsdq(); and (4) the
    panel quantile-regression slope homogeneity S-hat and D-hat
    statistics of Galvao, Juhl, Montes-Rojas and Olmo (2017)
    <doi:10.1080/07350015.2015.1054493> via xtqsh(). Together these
    tests address three fundamental pre-testing questions: (i) are
    slopes homogeneous? (ii) is there cross-sectional dependence? and
    (iii) is the panel balanced and is missingness ignorable?
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 3.5.0)
Imports: stats, utils
Suggests: testthat (>= 3.0.0), plm, zoo, quantreg
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-04-29 17:22:26 UTC; acad_
Author: Muhammad Abdullah Alkhalaf [aut, cre, cph] (ORCID:
    <https://orcid.org/0009-0002-2677-9246>)
Maintainer: Muhammad Abdullah Alkhalaf <muhammedalkhalaf@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-04 11:30:03 UTC
Built: R 4.5.2; ; 2026-05-04 12:25:19 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(print,xtcsdq)
S3method(print,xtqsh)
S3method(summary,xtcsdq)
S3method(summary,xtqsh)
export(xtcsdq)
export(xtmispanel)
export(xtpretest)
export(xtqsh)
importFrom(stats,complete.cases)

#' Comprehensive Panel Data Pre-Testing Suite
#'
#' Performs a full battery of panel data pre-tests: Hsiao (2014) homogeneity
#' F-tests, robust (HC1) versions, Swamy (1970) parameter heterogeneity test,
#' cross-sectional dependence (Pesaran 2004), and panel summary statistics.
#'
#' @param data A \code{data.frame} in long format containing all variables.
#' @param formula A two-sided formula of the form \code{y ~ x1 + x2 + ...}
#'   specifying the dependent and independent variables.
#' @param index Character vector of length 2: \code{c("panel_id", "time_id")}.
#' @param tests Character vector. Which modules to run. Possible values:
#'   \code{"summary"}, \code{"hsiao"}, \code{"robust"}, \code{"heterogeneity"},
#'   \code{"csd"}.  Default \code{"ALL"} runs everything.
#' @param level Numeric. Significance level for decisions (default 0.05).
#'
#' @return A list (invisibly) with components:
#'   \describe{
#'     \item{\code{summary}}{Panel summary statistics.}
#'     \item{\code{hsiao}}{Hsiao homogeneity F-test results.}
#'     \item{\code{robust}}{Robust HC1 F-test results.}
#'     \item{\code{swamy}}{Swamy heterogeneity test results.}
#'     \item{\code{csd}}{Cross-sectional dependence test results.}
#'     \item{\code{recommendation}}{Character. Suggested estimator.}
#'   }
#'
#' @references
#' Hsiao, C. (2014). \emph{Analysis of Panel Data} (3rd ed.).
#' Cambridge University Press.
#' \doi{10.1017/CBO9781139839327}
#'
#' Swamy, P. A. V. B. (1970). Efficient inference in a random coefficient
#' regression model. \emph{Econometrica}, 38(2), 311-323.
#' \doi{10.2307/1909405}
#'
#' Pesaran, M. H. (2004). General diagnostic tests for cross section
#' dependence in panels. \emph{Cambridge Working Paper in Economics}, No. 0435.
#' \doi{10.2139/ssrn.572504}
#'
#' @examples
#' set.seed(10)
#' n <- 5; t <- 10
#' df <- data.frame(
#'   id   = rep(1:n, each = t),
#'   time = rep(1:t, times = n),
#'   y    = rnorm(n * t),
#'   x1   = rnorm(n * t)
#' )
#' res <- xtpretest(df, y ~ x1, index = c("id", "time"),
#'                  tests = c("hsiao", "csd"))
#'
#' @export
xtpretest <- function(data,
                      formula,
                      index,
                      tests = "ALL",
                      level = 0.05) {

  ## ---- Validate --------------------------------------------------------
  if (!inherits(data, "data.frame")) stop("'data' must be a data.frame.")
  if (length(index) != 2L)          stop("'index' must be a character vector of length 2.")
  pid_col <- index[1L]; tid_col <- index[2L]
  if (!all(c(pid_col, tid_col) %in% names(data)))
    stop("index columns not found in 'data'.")

  tests <- toupper(tests)
  if (identical(tests, "ALL")) {
    tests <- c("SUMMARY", "HSIAO", "ROBUST", "HETEROGENEITY", "CSD")
  }

  ## ---- Extract variables -----------------------------------------------
  mf   <- stats::model.frame(formula, data = data, na.action = stats::na.pass)
  lhs  <- names(mf)[1L]
  rhs  <- names(mf)[-1L]
  y    <- data[[lhs]]
  X    <- data[, rhs, drop = FALSE]
  pid  <- data[[pid_col]]
  tid  <- data[[tid_col]]

  panels <- sort(unique(pid))
  N      <- length(panels)
  T_avg  <- nrow(data) / N
  K      <- length(rhs)

  .xtp_header(lhs, rhs, pid_col, tid_col, N, T_avg, nrow(data))

  out <- list()

  ## ---- MODULE: Summary ------------------------------------------------
  if ("SUMMARY" %in% tests) {
    out$summary <- .xtp_summary(data, formula, pid_col, tid_col, level)
  }

  ## ---- MODULE: Hsiao --------------------------------------------------
  if ("HSIAO" %in% tests) {
    out$hsiao <- .xtp_hsiao(y, X, pid, panels, K, level)
  }

  ## ---- MODULE: Robust -------------------------------------------------
  if ("ROBUST" %in% tests) {
    out$robust <- .xtp_robust(y, X, pid, data, pid_col, rhs, level)
  }

  ## ---- MODULE: Heterogeneity ------------------------------------------
  if ("HETEROGENEITY" %in% tests) {
    out$swamy <- .xtp_swamy(y, X, pid, panels, K)
  }

  ## ---- MODULE: CSD ----------------------------------------------------
  if ("CSD" %in% tests) {
    out$csd <- .xtp_csd_pesaran(y, X, pid, tid, panels, level)
  }

  ## ---- Recommendation -------------------------------------------------
  out$recommendation <- .xtp_recommend(out, level)
  .xtp_print_summary(out, level)

  invisible(out)
}


## ==========================================================================
## Internal helpers
## ==========================================================================

#' @keywords internal
.xtp_header <- function(lhs, rhs, pid, tid, N, T_avg, NT) {
  line <- strrep("-", 72)
  message(line)
  message("  xtpretest -- Comprehensive Panel Data Pre-Testing Suite v1.0.0")
  message(line)
  message("  Dependent  : ", lhs)
  message("  Regressors : ", paste(rhs, collapse = ", "))
  message("  Panel id   : ", pid, "  |  Time id: ", tid)
  message("  N panels   : ", N, "  |  Avg T: ", round(T_avg, 1),
          "  |  Obs: ", NT)
  message(line)
}

#' @keywords internal
.xtp_summary <- function(data, formula, pid_col, tid_col, level) {
  mf   <- stats::model.frame(formula, data = data, na.action = stats::na.pass)
  vars <- names(mf)
  pid  <- data[[pid_col]]
  out  <- list()

  line <- strrep("-", 72)
  message(line)
  message("  MODULE: Panel Summary Statistics")
  message(line)
  message(sprintf("  %-16s %-10s %-10s %-10s %-10s %-10s",
                  "Variable", "Overall SD", "Between SD", "Within SD", "Min", "Max"))
  message(line)

  for (v in vars) {
    x       <- data[[v]]
    o_sd    <- stats::sd(x, na.rm = TRUE)
    pmeans  <- tapply(x, pid, mean, na.rm = TRUE)
    b_sd    <- stats::sd(pmeans, na.rm = TRUE)
    grand   <- mean(x, na.rm = TRUE)
    within  <- x - pmeans[match(pid, names(pmeans))] + grand
    w_sd    <- stats::sd(within, na.rm = TRUE)
    message(sprintf("  %-16s %-10.4f %-10.4f %-10.4f %-10.4f %-10.4f",
                    utils::head(v, 1L),
                    o_sd, b_sd, w_sd,
                    min(x, na.rm = TRUE), max(x, na.rm = TRUE)))
    out[[v]] <- list(overall_sd = o_sd, between_sd = b_sd, within_sd = w_sd)
  }
  message(line)
  out
}

#' @keywords internal
.xtp_hsiao <- function(y, X, pid, panels, K, level) {
  N <- length(panels)
  ## S1: unrestricted (individual OLS)
  S1 <- 0; df1 <- 0
  for (p in panels) {
    idx  <- pid == p
    if (sum(idx) < K + 2L) next
    yi   <- y[idx]; Xi <- X[idx, , drop = FALSE]
    mat  <- cbind(1, as.matrix(Xi))
    fit  <- tryCatch(stats::lm.fit(mat, yi), error = function(e) NULL)
    if (!is.null(fit)) {
      S1  <- S1  + sum(fit$residuals^2)
      df1 <- df1 + (sum(idx) - K - 1L)
    }
  }

  ## S2: FE (common slopes, different intercepts)
  fe_mat <- cbind(stats::model.matrix(~ factor(pid) - 1), as.matrix(X))
  fit2   <- tryCatch(stats::lm.fit(fe_mat, y), error = function(e) NULL)
  S2     <- if (!is.null(fit2)) sum(fit2$residuals^2) else NA_real_
  df2    <- if (!is.null(fit2)) length(y) - ncol(fe_mat) else NA_integer_

  ## S3: pooled OLS
  mat3 <- cbind(1, as.matrix(X))
  fit3 <- tryCatch(stats::lm.fit(mat3, y), error = function(e) NULL)
  S3   <- if (!is.null(fit3)) sum(fit3$residuals^2) else NA_real_
  df3  <- if (!is.null(fit3)) length(y) - ncol(mat3) else NA_integer_

  ## F-statistics
  F1_nd <- (N - 1L) * (K + 1L); F1_dd <- df1
  F1  <- if (df1 > 0L && !is.na(S3)) ((S3 - S1) / F1_nd) / (S1 / F1_dd) else NA_real_
  F1p <- if (!is.na(F1)) stats::pf(F1, F1_nd, F1_dd, lower.tail = FALSE) else NA_real_

  F2_nd <- (N - 1L) * K; F2_dd <- df1
  F2  <- if (df1 > 0L && !is.na(S2)) ((S2 - S1) / F2_nd) / (S1 / F2_dd) else NA_real_
  F2p <- if (!is.na(F2)) stats::pf(F2, F2_nd, F2_dd, lower.tail = FALSE) else NA_real_

  F3_nd <- N - 1L; F3_dd <- df2
  F3  <- if (!is.na(df2) && df2 > 0L && !is.na(S3)) ((S3 - S2) / F3_nd) / (S2 / F3_dd) else NA_real_
  F3p <- if (!is.na(F3)) stats::pf(F3, F3_nd, F3_dd, lower.tail = FALSE) else NA_real_

  line <- strrep("-", 72)
  message(line)
  message("  MODULE: Hsiao (2014) Homogeneity Tests")
  message(line)
  message("  RSS Unrestricted (S1): ", formatC(S1, digits = 4, format = "f"))
  message("  RSS FE (S2):           ", formatC(S2, digits = 4, format = "f"))
  message("  RSS Pooled (S3):       ", formatC(S3, digits = 4, format = "f"))
  message(line)
  .xtp_print_ftest("F1: Overall homogeneity",   F1, F1_nd, F1_dd, F1p, level)
  .xtp_print_ftest("F2: Slope homogeneity",     F2, F2_nd, F2_dd, F2p, level)
  .xtp_print_ftest("F3: Intercept homogeneity", F3, F3_nd, F3_dd, F3p, level)
  message(line)

  list(F1 = F1, F1p = F1p, F1_nd = F1_nd, F1_dd = F1_dd,
       F2 = F2, F2p = F2p, F2_nd = F2_nd, F2_dd = F2_dd,
       F3 = F3, F3p = F3p, F3_nd = F3_nd, F3_dd = F3_dd,
       S1 = S1, S2 = S2, S3 = S3)
}

#' @keywords internal
.xtp_print_ftest <- function(label, F, df1, df2, pval, level) {
  sig <- if (!is.na(pval) && pval < level) " [REJECT]" else " [accept]"
  message(sprintf("  %-38s F=%8.4f  df=(%d,%d)  p=%6.4f%s",
                  label,
                  ifelse(is.na(F), 0, F),
                  df1, df2,
                  ifelse(is.na(pval), 1, pval),
                  sig))
}

#' @keywords internal
.xtp_robust <- function(y, X, pid, data, pid_col, rhs, level) {
  ## HC1 robust Wald tests via OLS with sandwich SE
  fe_dummy <- stats::model.matrix(~ factor(pid) - 1)
  int_terms <- do.call(cbind, lapply(rhs, function(v) {
    fe_dummy * data[[v]]
  }))
  colnames(int_terms) <- paste0("int_", seq_len(ncol(int_terms)))

  ## Full model
  full_mat <- cbind(1, as.matrix(X), fe_dummy[, -1L, drop = FALSE], int_terms)
  fit_full <- tryCatch(stats::lm.fit(full_mat, y), error = function(e) NULL)
  if (is.null(fit_full)) {
    message("  MODULE: Robust tests skipped (collinearity).")
    return(NULL)
  }

  ## HC1 sandwich
  hc1 <- .xtp_hc1(full_mat, fit_full$residuals)

  ## Robust F (Wald) for interaction terms (slope homogeneity)
  K_int <- ncol(int_terms)
  K_fe  <- ncol(fe_dummy) - 1L

  rF2 <- .xtp_wald(fit_full, hc1, seq(ncol(X) + 2L, ncol(X) + 1L + K_fe + K_int, 1L)[(K_fe + 1L):(K_fe + K_int)])
  rF3 <- .xtp_wald(fit_full, hc1, seq(ncol(X) + 2L, ncol(X) + 1L + K_fe, 1L))

  line <- strrep("-", 72)
  message(line)
  message("  MODULE: Robust (HC1) Homogeneity Tests")
  message(line)
  if (!is.null(rF2)) .xtp_print_ftest("Robust F2: Slope homogeneity", rF2$F, rF2$df1, rF2$df2, rF2$p, level)
  if (!is.null(rF3)) .xtp_print_ftest("Robust F3: Intercept homogeneity", rF3$F, rF3$df1, rF3$df2, rF3$p, level)
  message(line)

  list(rF2 = rF2, rF3 = rF3)
}

#' @keywords internal
.xtp_hc1 <- function(X, resid) {
  n <- nrow(X); k <- ncol(X)
  e2 <- resid^2 * (n / (n - k))
  Xe <- X * e2
  solve(crossprod(X)) %*% crossprod(Xe) %*% solve(crossprod(X))
}

#' @keywords internal
.xtp_wald <- function(fit, vcov, idx) {
  b   <- fit$coefficients[idx]
  V   <- vcov[idx, idx, drop = FALSE]
  df1 <- length(idx)
  df2 <- length(fit$residuals) - length(fit$coefficients)
  Wst <- tryCatch(as.numeric(t(b) %*% solve(V) %*% b) / df1,
                  error = function(e) NA_real_)
  pval <- if (!is.na(Wst)) stats::pf(Wst, df1, df2, lower.tail = FALSE) else NA_real_
  list(F = Wst, df1 = df1, df2 = df2, p = pval)
}

#' @keywords internal
.xtp_swamy <- function(y, X, pid, panels, K) {
  N <- length(panels)
  ## FE estimator
  fe_mat <- cbind(stats::model.matrix(~ factor(pid) - 1), as.matrix(X))
  fit_fe <- tryCatch(stats::lm.fit(fe_mat, y), error = function(e) NULL)
  if (is.null(fit_fe)) return(NULL)
  b_fe   <- utils::tail(fit_fe$coefficients, K)

  chi2 <- 0
  for (p in panels) {
    idx <- pid == p
    if (sum(idx) < K + 2L) next
    yi  <- y[idx]; Xi <- as.matrix(X[idx, , drop = FALSE])
    mat <- cbind(1, Xi)
    fit <- tryCatch(stats::lm(yi ~ Xi), error = function(e) NULL)
    if (is.null(fit)) next
    bi  <- stats::coef(fit)[-1L]
    Vi  <- stats::vcov(fit)[-1L, -1L, drop = FALSE]
    d   <- bi - b_fe
    chi2 <- chi2 + tryCatch(as.numeric(t(d) %*% solve(Vi) %*% d),
                             error = function(e) 0)
  }
  df   <- K * (N - 1L)
  pval <- stats::pchisq(chi2, df = df, lower.tail = FALSE)

  line <- strrep("-", 72)
  message(line)
  message("  MODULE: Swamy (1970) Parameter Heterogeneity")
  message(line)
  message(sprintf("  chi2(%d) = %.4f   p-value = %.4f  %s",
                  df, chi2, pval,
                  if (pval < 0.05) "[REJECT: heterogeneity]" else "[accept: homogeneity]"))
  message(line)

  list(chi2 = chi2, df = df, pval = pval)
}

#' @keywords internal
.xtp_csd_pesaran <- function(y, X, pid, tid, panels, level) {
  ## Run FE on the full model to get residuals
  fe_mat <- cbind(stats::model.matrix(~ factor(pid) - 1), as.matrix(X))
  fit_fe <- tryCatch(stats::lm.fit(fe_mat, y), error = function(e) NULL)
  if (is.null(fit_fe)) {
    message("  MODULE: CSD skipped.")
    return(NULL)
  }
  resid <- fit_fe$residuals
  N     <- length(panels)
  tvals <- sort(unique(tid))
  T_len <- length(tvals)

  ## Build residual matrix T x N
  R <- matrix(NA_real_, nrow = T_len, ncol = N)
  for (j in seq_along(panels)) {
    idx <- pid == panels[j]
    ti  <- match(tid[idx], tvals)
    R[ti, j] <- resid[idx]
  }

  ## Pairwise correlations
  cd_sum  <- 0; rho_sum <- 0; npairs <- 0
  for (i in seq_len(N - 1L)) {
    for (j in (i + 1L):N) {
      valid <- !is.na(R[, i]) & !is.na(R[, j])
      Tij   <- sum(valid)
      if (Tij < 3L) next
      rij   <- stats::cor(R[valid, i], R[valid, j])
      cd_sum  <- cd_sum  + sqrt(Tij) * rij
      rho_sum <- rho_sum + abs(rij)
      npairs  <- npairs + 1L
    }
  }
  cd_stat <- sqrt(2 / (N * (N - 1L))) * cd_sum
  pval    <- 2 * stats::pnorm(abs(cd_stat), lower.tail = FALSE)
  avg_rho <- rho_sum / max(npairs, 1L)

  line <- strrep("-", 72)
  message(line)
  message("  MODULE: Cross-Sectional Dependence (Pesaran 2004)")
  message(line)
  message(sprintf("  CD statistic   = %.4f", cd_stat))
  message(sprintf("  p-value        = %.4f", pval))
  message(sprintf("  Avg |rho_ij|   = %.4f", avg_rho))
  if (pval < level) {
    message("  -> Cross-sectional dependence detected. Consider CCE/CCEMG.")
  } else {
    message("  -> No significant cross-sectional dependence.")
  }
  message(line)

  list(cd_stat = cd_stat, pval = pval, avg_rho = avg_rho)
}

#' @keywords internal
.xtp_recommend <- function(out, level) {
  rec <- "Pooled OLS"

  if (!is.null(out$hsiao)) {
    if (!is.na(out$hsiao$F2p) && out$hsiao$F2p < level) rec <- "Mean Group (MG) / CCEMG"
    else if (!is.na(out$hsiao$F3p) && out$hsiao$F3p < level) rec <- "Fixed Effects (FE)"
  }
  if (!is.null(out$swamy)) {
    if (!is.na(out$swamy$pval) && out$swamy$pval < level && rec == "Pooled OLS")
      rec <- "Fixed Effects (FE)"
  }
  if (!is.null(out$csd)) {
    if (!is.na(out$csd$pval) && out$csd$pval < level) {
      if (rec == "Mean Group (MG) / CCEMG") rec <- "CCEMG (cross-sectional dependence)"
      else if (rec == "Fixed Effects (FE)") rec <- "FE with Driscoll-Kraay SE"
    }
  }
  rec
}

#' @keywords internal
.xtp_print_summary <- function(out, level) {
  line <- strrep("-", 72)
  message(line)
  message("  OVERALL PRE-TEST SUMMARY")
  message(line)
  if (!is.null(out$hsiao)) {
    message("  Hsiao F1 (overall)   : p=", round(out$hsiao$F1p, 3L))
    message("  Hsiao F2 (slopes)    : p=", round(out$hsiao$F2p, 3L))
    message("  Hsiao F3 (intercepts): p=", round(out$hsiao$F3p, 3L))
  }
  if (!is.null(out$swamy)) {
    message("  Swamy test           : p=", round(out$swamy$pval, 3L))
  }
  if (!is.null(out$csd)) {
    message("  CSD (Pesaran)        : p=", round(out$csd$pval, 3L))
  }
  message("  >>> Recommended estimator: ", out$recommendation)
  message(line)
}

make_panel <- function(n = 5, t = 10, seed = 1) {
  set.seed(seed)
  data.frame(
    id   = rep(seq_len(n), each = t),
    time = rep(seq_len(t), times = n),
    y    = rnorm(n * t),
    x1   = rnorm(n * t),
    x2   = rnorm(n * t)
  )
}

test_that("xtpretest runs Hsiao tests", {
  df  <- make_panel()
  res <- xtpretest(df, y ~ x1, index = c("id", "time"), tests = "HSIAO")
  expect_type(res, "list")
  expect_named(res$hsiao, c("F1", "F1p", "F1_nd", "F1_dd",
                             "F2", "F2p", "F2_nd", "F2_dd",
                             "F3", "F3p", "F3_nd", "F3_dd",
                             "S1", "S2", "S3"), ignore.order = TRUE)
  expect_true(is.numeric(res$hsiao$F1))
  expect_true(res$hsiao$F1p >= 0 && res$hsiao$F1p <= 1)
})

test_that("xtpretest runs CSD test", {
  df  <- make_panel(seed = 5)
  res <- xtpretest(df, y ~ x1 + x2, index = c("id", "time"), tests = "CSD")
  expect_named(res$csd, c("cd_stat", "pval", "avg_rho"), ignore.order = TRUE)
  expect_true(is.numeric(res$csd$cd_stat))
})

test_that("xtpretest returns a recommendation", {
  df  <- make_panel()
  res <- xtpretest(df, y ~ x1, index = c("id", "time"),
                   tests = c("HSIAO", "CSD"))
  expect_true(is.character(res$recommendation))
  expect_true(nchar(res$recommendation) > 0L)
})

test_that("xtpretest errors on bad index", {
  df <- make_panel()
  expect_error(xtpretest(df, y ~ x1, index = c("bad_id", "time")))
})

test_that("xtpretest summary module works", {
  df  <- make_panel()
  res <- xtpretest(df, y ~ x1, index = c("id", "time"), tests = "SUMMARY")
  expect_type(res$summary, "list")
})

#' Quantile Regression Slope Homogeneity Test for Panel Data
#'
#' Tests the null hypothesis of slope homogeneity in panel quantile regressions.
#' Implements the S-hat (chi-squared) and D-hat (standard normal) statistics of
#' Galvao, Juhl, Montes-Rojas and Olmo (2017).
#'
#' @param formula A formula of the form \code{y ~ x1 + x2 + ...} specifying the
#'   dependent variable and covariates.
#' @param data A data frame containing the panel data in long format.
#' @param index A character vector of length 2 giving the names of the panel
#'   individual and time variables, e.g. \code{c("id", "time")}.
#' @param tau A numeric vector of quantile levels, each strictly between 0 and 1.
#' @param bw Bandwidth method for sparsity estimation. Either
#'   \code{"hallsheather"} (default) or \code{"bofinger"}.
#' @param marginal Logical. If \code{TRUE}, per-variable marginal slope
#'   homogeneity tests are also computed. Default is \code{FALSE}.
#'
#' @return An object of class \code{"xtqsh"} with the following components:
#'   \describe{
#'     \item{S}{Numeric vector of S-hat statistics (one per quantile).}
#'     \item{D}{Numeric vector of D-hat statistics (one per quantile).}
#'     \item{pval_S}{p-values for S-hat (chi-squared distribution).}
#'     \item{pval_D}{p-values for D-hat (standard normal).}
#'     \item{S_ols}{S-hat statistic for the mean (OLS) regression.}
#'     \item{D_ols}{D-hat statistic for the mean (OLS) regression.}
#'     \item{pval_S_ols}{p-value for S_ols.}
#'     \item{pval_D_ols}{p-value for D_ols.}
#'     \item{beta_md}{Matrix of minimum-distance QR estimates (quantiles x regressors).}
#'     \item{beta_md_se}{Matrix of MD-QR standard errors.}
#'     \item{beta_all}{Array of per-panel QR coefficients (panels x regressors x quantiles).}
#'     \item{tau}{Quantile levels used.}
#'     \item{bw}{Bandwidth method used.}
#'     \item{n_panels}{Number of panel units.}
#'     \item{n_obs}{Total observations.}
#'     \item{k}{Number of regressors (excluding intercept).}
#'     \item{depvar}{Name of the dependent variable.}
#'     \item{indepvars}{Names of the independent variables.}
#'     \item{marginal}{List of marginal test results (if \code{marginal = TRUE}).}
#'   }
#'
#' @details
#' The test statistic D-hat is asymptotically standard normal under the null
#' hypothesis as both \eqn{N} and \eqn{T} tend to infinity. The S-hat statistic
#' follows a chi-squared distribution with \eqn{k(N-1)} degrees of freedom when
#' \eqn{T \to \infty} with \eqn{N} fixed.
#'
#' Bandwidth selection:
#' \itemize{
#'   \item \code{"hallsheather"}: \eqn{h = n^{-1/3} z_{(1+\tau)/2}^{2/3}
#'     \left(\frac{1.5 \phi^2(\Phi^{-1}(\tau))}{2(\Phi^{-1}(\tau))^2 + 1}\right)^{1/3}}
#'   \item \code{"bofinger"}: \eqn{h = n^{-1/5}
#'     \left(\frac{4.5 \phi^4(\Phi^{-1}(\tau))}{(2(\Phi^{-1}(\tau))^2 + 1)^2}\right)^{1/5}}
#' }
#'
#' @references
#' Galvao, A.F., Juhl, T., Montes-Rojas, G. and Olmo, J. (2017).
#' Testing Slope Homogeneity in Quantile Regression Panel Data with an
#' Application to the Cross-Section of Stock Returns.
#' \emph{Journal of Financial Econometrics}, 16(2), 211--243.
#' \doi{10.1093/jjfinec/nbx003}
#'
#' Bofinger, E. (1975). Estimation of a Density Function Using Order Statistics.
#' \emph{Australian Journal of Statistics}, 17(1), 1--7.
#'
#' Hall, P. and Sheather, S.J. (1988). On the Distribution of a Studentized
#' Quantile. \emph{Journal of the Royal Statistical Society Series B}, 50(3),
#' 381--391.
#'
#' @examples
#' \donttest{
#' set.seed(42)
#' n <- 10; tt <- 20
#' dat <- data.frame(
#'   id   = rep(1:n, each = tt),
#'   time = rep(1:tt, times = n),
#'   y    = rnorm(n * tt),
#'   x1   = rnorm(n * tt),
#'   x2   = rnorm(n * tt)
#' )
#' res <- xtqsh(y ~ x1 + x2, data = dat, index = c("id", "time"),
#'              tau = c(0.25, 0.5, 0.75))
#' print(res)
#' summary(res)
#' }
#'
#' @export
xtqsh <- function(formula, data, index, tau,
                  bw = "hallsheather", marginal = FALSE) {

  ## ── Input validation ────────────────────────────────────────────────────
  if (!inherits(formula, "formula")) {
    stop("'formula' must be a formula object.", call. = FALSE)
  }
  if (!is.data.frame(data)) {
    stop("'data' must be a data frame.", call. = FALSE)
  }
  if (!is.character(index) || length(index) != 2) {
    stop("'index' must be a character vector of length 2: c('id_var', 'time_var').",
         call. = FALSE)
  }
  if (!all(index %in% names(data))) {
    stop("Variables specified in 'index' not found in 'data'.", call. = FALSE)
  }
  if (!is.numeric(tau) || any(tau <= 0) || any(tau >= 1)) {
    stop("'tau' must be a numeric vector with all elements strictly between 0 and 1.",
         call. = FALSE)
  }
  tau <- sort(unique(tau))
  bw  <- match.arg(bw, c("hallsheather", "bofinger"))

  ## ── Parse formula ───────────────────────────────────────────────────────
  mf      <- stats::model.frame(formula, data = data, na.action = stats::na.omit)
  depvar  <- names(mf)[1]
  indvars <- names(mf)[-1]
  k       <- length(indvars)
  ntau    <- length(tau)

  if (k < 1) {
    stop("At least one independent variable is required.", call. = FALSE)
  }

  ivar <- index[1]
  tvar <- index[2]

  ## ── Identify panels ─────────────────────────────────────────────────────
  panels    <- sort(unique(data[[ivar]]))
  n_panels  <- length(panels)

  if (n_panels < 2) {
    stop("At least 2 panel units are required.", call. = FALSE)
  }

  n_obs <- nrow(mf)

  ## ── Per-panel QR estimation ──────────────────────────────────────────────
  # beta_all[i, j, q] = coefficient j for panel i at quantile tau[q]
  beta_all    <- array(NA_real_, dim = c(n_panels, k, ntau))
  beta_se_all <- array(NA_real_, dim = c(n_panels, k, ntau))
  valid_vec   <- logical(n_panels)

  for (qi in seq_along(tau)) {
    for (pi in seq_along(panels)) {
      idx   <- which(data[[ivar]] == panels[pi])
      subdf <- mf[rownames(mf) %in% rownames(data[idx, , drop = FALSE]),
                  , drop = FALSE]
      # fallback: match by position
      # Use a safe subset
      sub_data <- data[idx, , drop = FALSE]
      sub_mf   <- stats::model.frame(formula, data = sub_data,
                                     na.action = stats::na.omit)
      if (nrow(sub_mf) < k + 2) next

      Y_i <- sub_mf[[1]]
      X_i <- as.matrix(sub_mf[, -1, drop = FALSE])

      fit <- tryCatch(
        .qr_fit(Y_i, X_i, tau[qi]),
        error = function(e) NULL
      )
      if (is.null(fit)) next

      beta_all[pi, , qi] <- fit$coef
      beta_se_all[pi, , qi] <- fit$se
      valid_vec[pi] <- TRUE
    }
  }

  valid_panels <- sum(valid_vec)
  if (valid_panels < 2) {
    stop("Fewer than 2 panels produced valid QR estimates.", call. = FALSE)
  }

  ## ── OLS (mean) estimates per panel ──────────────────────────────────────
  beta_ols_all <- matrix(NA_real_, nrow = n_panels, ncol = k)
  for (pi in seq_along(panels)) {
    idx      <- which(data[[ivar]] == panels[pi])
    sub_data <- data[idx, , drop = FALSE]
    sub_mf   <- tryCatch(
      stats::model.frame(formula, data = sub_data, na.action = stats::na.omit),
      error = function(e) NULL
    )
    if (is.null(sub_mf) || nrow(sub_mf) < k + 2) next
    Y_i <- sub_mf[[1]]
    X_i <- as.matrix(sub_mf[, -1, drop = FALSE])
    ols <- tryCatch(
      stats::lm.fit(cbind(1, X_i), Y_i),
      error = function(e) NULL
    )
    if (!is.null(ols)) {
      beta_ols_all[pi, ] <- ols$coefficients[-1]
    }
  }

  ## ── Compute S-hat and D-hat for each quantile ───────────────────────────
  S_vec    <- numeric(ntau)
  D_vec    <- numeric(ntau)
  pval_S   <- numeric(ntau)
  pval_D   <- numeric(ntau)
  beta_md  <- matrix(NA_real_, nrow = ntau, ncol = k)
  beta_md_se <- matrix(NA_real_, nrow = ntau, ncol = k)

  for (qi in seq_along(tau)) {
    res <- .compute_slope_test(
      beta_all     = beta_all[valid_vec, , qi, drop = FALSE],
      beta_se_all  = beta_se_all[valid_vec, , qi, drop = FALSE],
      tau_val      = tau[qi],
      bw           = bw,
      n_panels_eff = valid_panels,
      k            = k
    )
    S_vec[qi]       <- res$S
    D_vec[qi]       <- res$D
    pval_S[qi]      <- res$pval_S
    pval_D[qi]      <- res$pval_D
    beta_md[qi, ]   <- res$beta_md
    beta_md_se[qi, ] <- res$beta_md_se
  }

  ## ── OLS (mean) joint test ───────────────────────────────────────────────
  ols_res <- .compute_ols_test(
    beta_ols_all = beta_ols_all[valid_vec, , drop = FALSE],
    k            = k,
    n_panels_eff = valid_panels
  )

  ## ── Marginal tests ───────────────────────────────────────────────────────
  marginal_res <- NULL
  if (marginal) {
    marginal_res <- list()
    for (j in seq_len(k)) {
      S_marg  <- numeric(ntau)
      D_marg  <- numeric(ntau)
      pS_marg <- numeric(ntau)
      pD_marg <- numeric(ntau)
      for (qi in seq_along(tau)) {
        b_j  <- beta_all[valid_vec, j, qi]
        se_j <- beta_se_all[valid_vec, j, qi]
        if (any(is.na(b_j)) || any(is.na(se_j))) {
          S_marg[qi] <- NA_real_; D_marg[qi] <- NA_real_
          pS_marg[qi] <- NA_real_; pD_marg[qi] <- NA_real_
          next
        }
        res_j <- .compute_slope_test(
          beta_all     = array(b_j, dim = c(valid_panels, 1, 1)),
          beta_se_all  = array(se_j, dim = c(valid_panels, 1, 1)),
          tau_val      = tau[qi],
          bw           = bw,
          n_panels_eff = valid_panels,
          k            = 1L
        )
        S_marg[qi]  <- res_j$S
        D_marg[qi]  <- res_j$D
        pS_marg[qi] <- res_j$pval_S
        pD_marg[qi] <- res_j$pval_D
      }
      marginal_res[[indvars[j]]] <- list(
        S = S_marg, D = D_marg, pval_S = pS_marg, pval_D = pD_marg
      )
    }
  }

  ## ── Assemble output ──────────────────────────────────────────────────────
  colnames(beta_md)    <- indvars
  colnames(beta_md_se) <- indvars
  rownames(beta_md)    <- paste0("tau=", tau)
  rownames(beta_md_se) <- paste0("tau=", tau)

  out <- list(
    S           = S_vec,
    D           = D_vec,
    pval_S      = pval_S,
    pval_D      = pval_D,
    S_ols       = ols_res$S,
    D_ols       = ols_res$D,
    pval_S_ols  = ols_res$pval_S,
    pval_D_ols  = ols_res$pval_D,
    beta_md     = beta_md,
    beta_md_se  = beta_md_se,
    beta_all    = beta_all,
    beta_ols    = beta_ols_all,
    tau         = tau,
    bw          = bw,
    n_panels    = valid_panels,
    n_obs       = n_obs,
    k           = k,
    depvar      = depvar,
    indepvars   = indvars,
    marginal    = marginal_res
  )
  class(out) <- "xtqsh"
  out
}


## ── Internal: quantile regression via simplex / IRLS ─────────────────────
#' @keywords internal
.qr_fit <- function(y, X, tau) {
  n  <- length(y)
  Xm <- cbind(1, X)
  p  <- ncol(Xm)

  ## Barrodale-Roberts simplex (using stats::quantreg unavailable;
  ## implement a lightweight version via iteratively weighted LS)
  coef <- .qr_irls(y, Xm, tau)

  ## Sparsity (sandwich) SE estimate
  resid <- y - Xm %*% coef
  se    <- .qr_se(y, Xm, resid, tau)

  list(coef = coef[-1], se = se[-1])   # drop intercept
}


#' @keywords internal
.qr_irls <- function(y, X, tau, maxit = 200L, tol = 1e-8) {
  n <- length(y)
  p <- ncol(X)
  beta <- stats::lm.fit(X, y)$coefficients

  for (iter in seq_len(maxit)) {
    resid  <- y - X %*% beta
    w      <- ifelse(abs(resid) < tol, tol, abs(resid))
    w      <- ifelse(resid >= 0, tau / w, (1 - tau) / w)
    W      <- diag(as.numeric(w), n)
    XtWX   <- crossprod(X, W %*% X)
    XtWy   <- crossprod(X, W %*% y)
    beta_new <- tryCatch(
      solve(XtWX, XtWy),
      error = function(e) beta
    )
    if (max(abs(beta_new - beta)) < tol) {
      beta <- beta_new
      break
    }
    beta <- beta_new
  }
  as.numeric(beta)
}


#' @keywords internal
.qr_se <- function(y, X, resid, tau) {
  n  <- length(y)
  p  <- ncol(X)
  h  <- .bw_hallsheather(n, tau)
  f_hat <- mean(stats::dnorm(resid / h) / h)
  if (f_hat < 1e-10) f_hat <- 1e-10
  J    <- crossprod(X) / n
  Jinv <- tryCatch(solve(J), error = function(e) diag(p))
  V    <- tau * (1 - tau) / (n * f_hat^2) * Jinv %*% crossprod(X) %*% Jinv
  sqrt(pmax(diag(V), 0))
}


## ── Bandwidth helpers ────────────────────────────────────────────────────
#' @keywords internal
.bw_hallsheather <- function(n, tau) {
  z  <- stats::qnorm((1 + tau) / 2)
  phi <- stats::dnorm(stats::qnorm(tau))
  n^(-1/3) * z^(2/3) * ((1.5 * phi^2) / (2 * stats::qnorm(tau)^2 + 1))^(1/3)
}

#' @keywords internal
.bw_bofinger <- function(n, tau) {
  phi <- stats::dnorm(stats::qnorm(tau))
  z   <- stats::qnorm(tau)
  n^(-1/5) * ((4.5 * phi^4) / (2 * z^2 + 1)^2)^(1/5)
}


## ── Test statistic computation ───────────────────────────────────────────
#' @keywords internal
.compute_slope_test <- function(beta_all, beta_se_all, tau_val, bw,
                                 n_panels_eff, k) {
  # beta_all: [n_panels_eff, k, 1] or [n_panels_eff, k]
  if (length(dim(beta_all)) == 3) {
    beta_all    <- matrix(beta_all[, , 1], nrow = dim(beta_all)[1], ncol = dim(beta_all)[2])
    beta_se_all <- matrix(beta_se_all[, , 1], nrow = dim(beta_se_all)[1], ncol = dim(beta_se_all)[2])
  }
  beta_all <- as.matrix(beta_all)
  beta_se_all <- as.matrix(beta_se_all)

  if (any(is.na(beta_all)) || any(is.na(beta_se_all))) {
    return(list(S = NA_real_, D = NA_real_,
                pval_S = NA_real_, pval_D = NA_real_,
                beta_md = rep(NA_real_, k),
                beta_md_se = rep(NA_real_, k)))
  }

  N <- n_panels_eff

  ## Minimum distance (pooled) estimator: weighted mean
  ## V_i^{-1} = diag(1/se_i^2) — diagonal weight matrix
  beta_md  <- numeric(k)
  beta_md_se_v <- numeric(k)
  for (j in seq_len(k)) {
    w_j       <- 1 / (beta_se_all[, j, drop = FALSE]^2 + 1e-14)
    beta_md[j] <- sum(w_j * beta_all[, j]) / sum(w_j)
    beta_md_se_v[j] <- sqrt(1 / sum(w_j))
  }

  ## S-hat statistic (chi-squared, k*(N-1) df)
  ## S = sum_i (beta_i - beta_md)' V_i^{-1} (beta_i - beta_md)
  S_val <- 0
  for (i in seq_len(N)) {
    diff_i <- beta_all[i, ] - beta_md
    Vi_inv  <- diag(1 / (beta_se_all[i, ]^2 + 1e-14), k)
    S_val   <- S_val + as.numeric(t(diff_i) %*% Vi_inv %*% diff_i)
  }

  df     <- k * (N - 1)
  pval_S <- 1 - stats::pchisq(S_val, df = df)

  ## D-hat statistic (standard normal)
  ## D = (S - k*(N-1)) / sqrt(2*k*(N-1))
  D_val  <- (S_val - k * (N - 1)) / sqrt(2 * k * (N - 1))
  pval_D <- 1 - stats::pnorm(D_val)

  list(S = S_val, D = D_val, pval_S = pval_S, pval_D = pval_D,
       beta_md = beta_md, beta_md_se = beta_md_se_v)
}


#' @keywords internal
.compute_ols_test <- function(beta_ols_all, k, n_panels_eff) {
  valid <- complete.cases(beta_ols_all)
  beta  <- beta_ols_all[valid, , drop = FALSE]
  N     <- nrow(beta)
  if (N < 2) {
    return(list(S = NA_real_, D = NA_real_, pval_S = NA_real_, pval_D = NA_real_))
  }
  beta_mean <- colMeans(beta)
  # Use sample variance across panels as weighting proxy
  S_val <- 0
  for (i in seq_len(N)) {
    diff_i <- beta[i, ] - beta_mean
    S_val  <- S_val + sum(diff_i^2)
  }
  # Standardize by average within-panel variance (set to 1 for OLS pooled test)
  df     <- k * (N - 1)
  pval_S <- 1 - stats::pchisq(S_val, df = df)
  D_val  <- (S_val - k * (N - 1)) / sqrt(2 * k * (N - 1))
  pval_D <- 1 - stats::pnorm(D_val)
  list(S = S_val, D = D_val, pval_S = pval_S, pval_D = pval_D)
}


## ── S3 methods ───────────────────────────────────────────────────────────

#' Print method for xtqsh objects
#'
#' @param x An object of class \code{"xtqsh"}.
#' @param ... Additional arguments (ignored).
#' @return Invisibly returns \code{x}.
#' @export
print.xtqsh <- function(x, ...) {
  cat("\n")
  cat(strrep("-", 70), "\n")
  cat("  Quantile Regression Slope Homogeneity Test\n")
  cat("  Galvao, Juhl, Montes-Rojas & Olmo (2017)\n")
  cat(strrep("-", 70), "\n")
  cat(sprintf("  Dep. variable : %s\n", x$depvar))
  cat(sprintf("  Covariates    : %s\n", paste(x$indepvars, collapse = ", ")))
  cat(sprintf("  Observations  : %d\n", x$n_obs))
  cat(sprintf("  Panels (valid): %d\n", x$n_panels))
  cat(sprintf("  Quantiles     : %s\n", paste(x$tau, collapse = ", ")))
  cat(sprintf("  Bandwidth     : %s\n", x$bw))
  cat(strrep("-", 70), "\n\n")

  cat("  Joint Slope Homogeneity Test\n")
  cat(strrep("-", 70), "\n")
  cat(sprintf("  %-10s  %12s  %10s  %12s  %10s\n",
              "Quantile", "S-hat", "p(S)", "D-hat", "p(D)"))
  cat(strrep("-", 70), "\n")

  cat(sprintf("  %-10s  %12.4f  %10.4f  %12.4f  %10.4f  %s\n",
              "Mean(OLS)", x$S_ols, x$pval_S_ols,
              x$D_ols, x$pval_D_ols,
              .stars(x$pval_D_ols)))

  for (i in seq_along(x$tau)) {
    cat(sprintf("  tau=%-6.2f  %12.4f  %10.4f  %12.4f  %10.4f  %s\n",
                x$tau[i], x$S[i], x$pval_S[i],
                x$D[i], x$pval_D[i],
                .stars(x$pval_D[i])))
  }
  cat(strrep("-", 70), "\n")
  cat("  *** p<0.01, ** p<0.05, * p<0.10\n\n")

  invisible(x)
}


#' Summary method for xtqsh objects
#'
#' @param object An object of class \code{"xtqsh"}.
#' @param ... Additional arguments (ignored).
#' @return Invisibly returns \code{object}.
#' @export
summary.xtqsh <- function(object, ...) {
  print(object, ...)

  cat("  Minimum Distance QR Estimates\n")
  cat(strrep("-", 70), "\n")
  print(round(object$beta_md, 4))
  cat("  Standard errors:\n")
  print(round(object$beta_md_se, 4))
  cat("\n")

  if (!is.null(object$marginal)) {
    cat("  Marginal Slope Homogeneity Tests\n")
    cat(strrep("-", 70), "\n")
    for (vname in names(object$marginal)) {
      cat(sprintf("  Variable: %s\n", vname))
      mr <- object$marginal[[vname]]
      for (i in seq_along(object$tau)) {
        cat(sprintf("    tau=%.2f  S=%.4f  p(S)=%.4f  D=%.4f  p(D)=%.4f  %s\n",
                    object$tau[i], mr$S[i], mr$pval_S[i],
                    mr$D[i], mr$pval_D[i], .stars(mr$pval_D[i])))
      }
    }
  }

  invisible(object)
}


#' @keywords internal
.stars <- function(p) {
  if (is.na(p)) return("")
  if (p < 0.01) return("***")
  if (p < 0.05) return("**")
  if (p < 0.10) return("*")
  return("")
}

test_that("xtqsh returns correct structure", {
  set.seed(123)
  n <- 8; tt <- 15
  dat <- data.frame(
    id   = rep(1:n, each = tt),
    time = rep(1:tt, times = n),
    y    = rnorm(n * tt),
    x1   = rnorm(n * tt)
  )
  res <- xtqsh(y ~ x1, data = dat, index = c("id", "time"), tau = c(0.25, 0.5, 0.75))
  expect_s3_class(res, "xtqsh")
  expect_length(res$tau, 3)
  expect_length(res$S, 3)
  expect_length(res$D, 3)
  expect_true(all(!is.na(res$pval_D)))
  expect_true(all(res$pval_D >= 0 & res$pval_D <= 1))
})

test_that("xtqsh beta_md has correct dimensions", {
  set.seed(42)
  n <- 6; tt <- 20
  dat <- data.frame(
    id   = rep(1:n, each = tt),
    time = rep(1:tt, times = n),
    y    = rnorm(n * tt),
    x1   = rnorm(n * tt),
    x2   = rnorm(n * tt)
  )
  res <- xtqsh(y ~ x1 + x2, data = dat, index = c("id", "time"), tau = 0.5)
  expect_equal(dim(res$beta_md), c(1L, 2L))
  expect_equal(colnames(res$beta_md), c("x1", "x2"))
})

test_that("xtqsh marginal tests work", {
  set.seed(7)
  n <- 6; tt <- 15
  dat <- data.frame(
    id   = rep(1:n, each = tt),
    time = rep(1:tt, times = n),
    y    = rnorm(n * tt),
    x1   = rnorm(n * tt)
  )
  res <- xtqsh(y ~ x1, data = dat, index = c("id", "time"),
               tau = c(0.25, 0.75), marginal = TRUE)
  expect_false(is.null(res$marginal))
  expect_true("x1" %in% names(res$marginal))
})

test_that("print and summary do not error", {
  set.seed(1)
  n <- 5; tt <- 12
  dat <- data.frame(
    id   = rep(1:n, each = tt),
    time = rep(1:tt, times = n),
    y    = rnorm(n * tt),
    x1   = rnorm(n * tt)
  )
  res <- xtqsh(y ~ x1, data = dat, index = c("id", "time"), tau = 0.5)
  expect_output(print(res))
  expect_output(summary(res))
})

#' Missing Data Detection and Imputation for Panel Data
#'
#' Detects, diagnoses, and imputes missing values in panel (longitudinal)
#' data sets.  The function can produce summary tables (Module 1), test the
#' missingness mechanism (Module 2), impute a target variable (Module 3),
#' and run a cross-method sensitivity analysis (Module 4).
#'
#' @param data A \code{data.frame} in long format.
#' @param vars Character vector of variable names to analyse.  If \code{NULL}
#'   (default), all numeric columns except the index are used.
#' @param index Character vector of length 2: \code{c("panel_id", "time_id")}.
#' @param detect Logical. Run Module 1 (detection tables, default \code{TRUE}).
#' @param test Logical. Run Module 2 (MCAR/MAR mechanism tests,
#'   default \code{FALSE}).
#' @param impute Character or \code{NULL}. If a method name is given, run
#'   Module 3 (imputation).  Supported methods:
#'   \code{"mean"}, \code{"median"}, \code{"locf"}, \code{"nocb"},
#'   \code{"linear"}, \code{"spline"}, \code{"pmm"}, \code{"hotdeck"},
#'   \code{"knn"}, \code{"rf"}, \code{"em"}.
#' @param target Character. Name of the variable to impute (required when
#'   \code{impute} is not \code{NULL}).
#' @param new_var Character. Name of the output imputed variable
#'   (default \code{"\{target\}_imp"}).
#' @param sensitivity Logical. Run Module 4 (sensitivity analysis across
#'   all imputation methods, default \code{FALSE}).
#' @param knn_k Integer. Number of neighbours for KNN imputation (default 5).
#'
#' @return A list (invisibly) with components:
#'   \describe{
#'     \item{\code{detect}}{Summary statistics per variable/panel/period.}
#'     \item{\code{test}}{MCAR and MAR test results.}
#'     \item{\code{imputed}}{The \code{data} frame augmented with the imputed
#'       column (when imputation is requested).}
#'     \item{\code{impute_stats}}{Summary comparing original vs imputed.}
#'     \item{\code{sensitivity}}{Sensitivity analysis results.}
#'   }
#'
#' @references
#' Little, R. J. A. (1988). A test of missing completely at random for
#' multivariate data with missing values.
#' \emph{Journal of the American Statistical Association}, 83(404), 1198-1202.
#' \doi{10.1080/01621459.1988.10478714}
#'
#' @examples
#' set.seed(1)
#' df <- data.frame(
#'   id   = rep(1:4, each = 8),
#'   time = rep(1:8, times = 4),
#'   y    = c(rnorm(32))
#' )
#' # introduce some NAs
#' df$y[c(3, 11, 20)] <- NA
#' res <- xtmispanel(df, vars = "y", index = c("id", "time"), detect = TRUE)
#'
#' @export
xtmispanel <- function(data,
                       vars        = NULL,
                       index,
                       detect      = TRUE,
                       test        = FALSE,
                       impute      = NULL,
                       target      = NULL,
                       new_var     = NULL,
                       sensitivity = FALSE,
                       knn_k       = 5L) {

  ## ---- Validate --------------------------------------------------------
  if (!inherits(data, "data.frame")) stop("'data' must be a data.frame.")
  if (length(index) != 2L) stop("'index' must be a character vector of length 2.")
  pid_col <- index[1L]; tid_col <- index[2L]
  if (!all(c(pid_col, tid_col) %in% names(data)))
    stop("index columns not found in 'data'.")

  ## Determine analysis variables
  if (is.null(vars)) {
    num_cols <- vapply(data, is.numeric, logical(1L))
    vars <- setdiff(names(data)[num_cols], c(pid_col, tid_col))
  }
  vars <- intersect(vars, names(data))
  if (length(vars) == 0L) stop("No valid numeric variables found.")

  pid <- data[[pid_col]]
  tid <- data[[tid_col]]

  .xmp_header(pid_col, tid_col, pid, tid, data)
  out <- list()

  ## ---- Module 1: Detection --------------------------------------------
  if (detect) {
    out$detect <- .xmp_detect(data, vars, pid, tid, pid_col, tid_col)
  }

  ## ---- Module 2: Mechanism tests --------------------------------------
  if (test) {
    out$test <- .xmp_tests(data, vars, pid, tid)
  }

  ## ---- Module 3: Imputation -------------------------------------------
  if (!is.null(impute)) {
    if (is.null(target) || !target %in% names(data))
      stop("'target' must be the name of a variable in 'data'.")
    if (is.null(new_var)) new_var <- paste0(target, "_imp")
    res_imp <- .xmp_impute(data, target, pid, tid, method = impute, knn_k = knn_k)
    data[[new_var]] <- res_imp
    out$imputed      <- data
    out$impute_stats <- .xmp_compare(data[[target]], data[[new_var]],
                                     target, new_var, impute)
  }

  ## ---- Module 4: Sensitivity ------------------------------------------
  if (sensitivity) {
    if (is.null(target) || !target %in% names(data))
      stop("'target' must be set for sensitivity analysis.")
    out$sensitivity <- .xmp_sensitivity(data, target, pid, tid, knn_k = knn_k)
  }

  invisible(out)
}


## ==========================================================================
## Internal helpers
## ==========================================================================

#' @keywords internal
.xmp_header <- function(pid_col, tid_col, pid, tid, data) {
  N  <- length(unique(pid))
  T2 <- length(unique(tid))
  message(strrep("-", 72))
  message("  xtmispanel -- Missing Data Diagnostics for Panel Data  v1.0.0")
  message(strrep("-", 72))
  message("  Panel id: ", pid_col, "  |  Time id: ", tid_col)
  message("  Panels: ", N, "  |  Time periods: ", T2,
          "  |  Rows: ", nrow(data))
  message(strrep("-", 72))
}

#' @keywords internal
.xmp_detect <- function(data, vars, pid, tid, pid_col, tid_col) {
  line <- strrep("-", 72)
  message(line)
  message("  TABLE 1: Missing Data Summary by Variable")
  message(line)
  message(sprintf("  %-16s %8s %8s %7s %10s %10s  %s",
                  "Variable", "N_Total", "N_Miss", "%Miss", "Mean", "SD", "Status"))
  message(line)

  out_vars <- list()
  total_miss <- 0L; total_obs <- 0L

  for (v in vars) {
    x     <- data[[v]]
    ntot  <- length(x)
    nmiss <- sum(is.na(x))
    pct   <- if (ntot > 0) nmiss / ntot * 100 else 0
    vmean <- mean(x, na.rm = TRUE)
    vsd   <- stats::sd(x, na.rm = TRUE)
    if (is.na(vsd)) vsd <- 0
    status <- if (pct == 0) "Complete"
    else if (pct <= 5) "Low"
    else if (pct <= 20) "Moderate"
    else if (pct <= 50) "High"
    else "Severe"
    message(sprintf("  %-16s %8d %8d %6.1f%% %10.3f %10.3f  %s",
                    substr(v, 1L, 16L), ntot, nmiss, pct, vmean, vsd, status))
    out_vars[[v]] <- list(n = ntot, n_miss = nmiss, pct = pct,
                          mean = vmean, sd = vsd, status = status)
    total_miss <- total_miss + nmiss
    total_obs  <- total_obs  + ntot
  }
  message(line)
  message(sprintf("  %-16s %8d %8d %6.1f%%",
                  "Overall", total_obs, total_miss,
                  if (total_obs > 0) total_miss / total_obs * 100 else 0))
  message(line)

  ## Per-panel summary
  panels  <- sort(unique(pid))
  message(line)
  message("  TABLE 2: Missing Data by Panel")
  message(line)
  message(sprintf("  %-14s %8s %8s %7s  %s",
                  "Panel", "N_Obs", "N_Miss", "%Miss", "Status"))
  message(line)
  out_panels <- list()
  for (p in panels) {
    idx   <- pid == p
    pmiss <- sum(is.na(unlist(data[idx, vars, drop = FALSE])))
    pobs  <- sum(idx) * length(vars)
    ppct  <- if (pobs > 0) pmiss / pobs * 100 else 0
    pstat <- if (ppct == 0) "Complete"
    else if (ppct <= 10) "Low"
    else if (ppct <= 30) "Moderate"
    else "High"
    message(sprintf("  %-14s %8d %8d %6.1f%%  %s",
                    substr(as.character(p), 1L, 14L), pobs, pmiss, ppct, pstat))
    out_panels[[as.character(p)]] <- list(pobs = pobs, pmiss = pmiss, pct = ppct)
  }
  message(line)

  list(variables = out_vars, panels = out_panels,
       total_missing = total_miss, total_obs = total_obs)
}

#' @keywords internal
.xmp_tests <- function(data, vars, pid, tid) {
  line <- strrep("-", 72)
  message(line)
  message("  MODULE 2: Missing Data Mechanism Tests")
  message(line)

  ## Approximate Little MCAR test
  message("  Test 1: Approximate Little MCAR Test")
  chi2_total <- 0; df_total <- 0

  for (v in vars) {
    if (sum(is.na(data[[v]])) == 0L) next
    gmean <- mean(data[[v]], na.rm = TRUE)
    gvar  <- stats::var(data[[v]], na.rm = TRUE)
    if (is.na(gvar) || gvar == 0) next
    for (w in vars) {
      if (v == w) next
      if (sum(is.na(data[[w]])) == 0L) next
      m_miss <- mean(data[[v]][is.na(data[[w]])], na.rm = TRUE)
      m_obs  <- mean(data[[v]][!is.na(data[[w]])], na.rm = TRUE)
      n1 <- sum(is.na(data[[w]]) & !is.na(data[[v]]))
      n2 <- sum(!is.na(data[[w]]) & !is.na(data[[v]]))
      if (n1 < 2L || n2 < 2L) next
      se <- sqrt(gvar * (1 / n1 + 1 / n2))
      if (se > 0) {
        z <- (m_miss - m_obs) / se
        chi2_total <- chi2_total + z^2
        df_total   <- df_total + 1L
      }
    }
  }

  mcar_result <- NULL
  if (df_total > 0L) {
    pval <- stats::pchisq(chi2_total, df = df_total, lower.tail = FALSE)
    message(sprintf("  chi2(%d) = %.3f   p = %.4f   %s",
                    df_total, chi2_total, pval,
                    if (pval < 0.05) "REJECT H0: NOT MCAR" else "Fail to reject: possibly MCAR"))
    mcar_result <- list(chi2 = chi2_total, df = df_total, pval = pval)
  } else {
    message("  Test could not be computed (insufficient variation).")
  }

  ## MAR logistic test
  message(line)
  message("  Test 2: MAR Logistic Regression Test")
  message(sprintf("  %-16s %10s %10s %10s  %s",
                  "Variable", "chi2", "p-value", "Pseudo-R2", "Conclusion"))
  message(line)

  mar_results <- list()
  for (v in vars) {
    nmv <- sum(is.na(data[[v]]))
    if (nmv == 0L) { message(sprintf("  %-16s %10s", substr(v,1,16), "No missing")); next }
    mis_ind <- as.integer(is.na(data[[v]]))
    preds   <- setdiff(vars, v)
    if (length(preds) == 0L) next
    pred_df <- data[, preds, drop = FALSE]
    complete_rows <- stats::complete.cases(pred_df)
    if (sum(complete_rows) < length(preds) + 2L) next
    fit <- tryCatch(
      stats::glm(mis_ind[complete_rows] ~ as.matrix(pred_df[complete_rows, ]),
                 family = stats::binomial()),
      error = function(e) NULL
    )
    if (is.null(fit)) { message(sprintf("  %-16s  FAILED", substr(v,1,16))); next }
    null_dev <- fit$null.deviance; res_dev <- fit$deviance
    chi2  <- null_dev - res_dev
    df_m  <- fit$df.null - fit$df.residual
    pval  <- stats::pchisq(chi2, df = df_m, lower.tail = FALSE)
    pr2   <- 1 - res_dev / null_dev
    conc  <- if (!is.na(pval) && pval < 0.05) "MAR" else "MCAR"
    message(sprintf("  %-16s %10.3f %10.4f %10.4f  %s",
                    substr(v, 1L, 16L), chi2, pval, pr2, conc))
    mar_results[[v]] <- list(chi2 = chi2, df = df_m, pval = pval, conclusion = conc)
  }
  message(line)
  list(mcar = mcar_result, mar = mar_results)
}

#' @keywords internal
.xmp_impute <- function(data, target, pid, tid, method, knn_k) {
  x       <- data[[target]]
  out     <- x
  panels  <- sort(unique(pid))

  method <- tolower(trimws(method))

  for (p in panels) {
    idx  <- which(pid == p)
    vals <- x[idx]
    ord  <- order(tid[idx])
    vals <- vals[ord]

    vals_imp <- switch(method,
      mean    = { m <- mean(vals, na.rm = TRUE); ifelse(is.na(vals), m, vals) },
      median  = { m <- stats::median(vals, na.rm = TRUE); ifelse(is.na(vals), m, vals) },
      locf    = .xmp_locf(vals),
      nocb    = rev(.xmp_locf(rev(vals))),
      linear  = .xmp_linear_interp(vals),
      spline  = .xmp_spline_interp(vals),
      pmm     = .xmp_pmm(vals),
      hotdeck = .xmp_hotdeck(vals),
      knn     = .xmp_knn(data, target, pid, tid, p, knn_k),
      rf      = .xmp_rf(data, target, pid, tid, p),
      em      = .xmp_em(vals),
      {
        message("  xtmispanel: unknown method '", method, "'; using mean.")
        m <- mean(vals, na.rm = TRUE); ifelse(is.na(vals), m, vals)
      }
    )
    out[idx[ord]] <- vals_imp
  }
  out
}

## --- Imputation method implementations ---

#' @keywords internal
.xmp_locf <- function(x) {
  for (i in seq_along(x)) {
    if (is.na(x[i]) && i > 1L) x[i] <- x[i - 1L]
  }
  x
}

#' @keywords internal
.xmp_linear_interp <- function(x) {
  n   <- length(x)
  idx <- which(!is.na(x))
  if (length(idx) < 2L) return(x)
  stats::approx(idx, x[idx], xout = seq_len(n), rule = 2L)$y
}

#' @keywords internal
.xmp_spline_interp <- function(x) {
  n   <- length(x)
  idx <- which(!is.na(x))
  if (length(idx) < 3L) return(.xmp_linear_interp(x))
  tryCatch(
    stats::spline(idx, x[idx], xout = seq_len(n), method = "natural")$y,
    error = function(e) .xmp_linear_interp(x)
  )
}

#' @keywords internal
.xmp_pmm <- function(x) {
  obs_vals <- x[!is.na(x)]
  if (length(obs_vals) == 0L) return(x)
  out <- x
  for (i in which(is.na(x))) {
    i_ctx <- mean(obs_vals)  # simplified: use global mean as donor context
    dists <- abs(obs_vals - i_ctx)
    out[i] <- obs_vals[which.min(dists)]
  }
  out
}

#' @keywords internal
.xmp_hotdeck <- function(x) {
  obs_vals <- x[!is.na(x)]
  if (length(obs_vals) == 0L) return(x)
  out <- x
  miss_idx <- which(is.na(x))
  out[miss_idx] <- sample(obs_vals, length(miss_idx), replace = TRUE)
  out
}

#' @keywords internal
.xmp_knn <- function(data, target, pid, tid, panel, k) {
  idx      <- which(pid == panel)
  vals     <- data[[target]][idx]
  ord      <- order(tid[idx])
  vals     <- vals[ord]
  miss_pos <- which(is.na(vals))
  if (length(miss_pos) == 0L) return(vals)

  # Use other panels' values at same time positions as donors
  other_idx <- which(pid != panel)
  donors    <- data[[target]][other_idx]
  donors    <- donors[!is.na(donors)]
  if (length(donors) == 0L) donors <- stats::na.omit(vals)
  if (length(donors) == 0L) return(vals)

  obs_vals <- vals[!is.na(vals)]
  for (i in miss_pos) {
    # time-position context: average of window
    window <- vals[max(1L, i - 2L):min(length(vals), i + 2L)]
    ctx    <- mean(window, na.rm = TRUE)
    if (is.na(ctx)) ctx <- mean(donors)
    dists  <- abs(donors - ctx)
    nn     <- utils::head(order(dists), k)
    vals[i] <- mean(donors[nn])
  }
  vals
}

#' @keywords internal
.xmp_rf <- function(data, target, pid, tid, panel) {
  ## Simplified random-forest-like imputation via bagged linear models
  idx      <- which(pid == panel)
  vals     <- data[[target]][idx]
  ord      <- order(tid[idx])
  vals     <- vals[ord]
  miss_pos <- which(is.na(vals))
  if (length(miss_pos) == 0L) return(vals)

  obs_idx  <- which(!is.na(vals))
  if (length(obs_idx) < 3L) return(.xmp_linear_interp(vals))

  t_idx <- seq_along(vals)
  preds <- numeric(length(miss_pos))
  B <- 20L
  for (b in seq_len(B)) {
    samp <- sample(obs_idx, replace = TRUE)
    fit  <- tryCatch(stats::lm(vals[samp] ~ t_idx[samp]),
                     error = function(e) NULL)
    if (!is.null(fit)) {
      preds <- preds + stats::predict(fit, newdata = data.frame(`t_idx[samp]` = t_idx[miss_pos]))
    }
  }
  vals[miss_pos] <- preds / B
  vals
}

#' @keywords internal
.xmp_em <- function(x) {
  ## Single-variable EM: iterate between mean estimate and imputation
  obs  <- x[!is.na(x)]
  if (length(obs) == 0L) return(x)
  mu   <- mean(obs); sig2 <- stats::var(obs)
  if (is.na(sig2) || sig2 == 0) sig2 <- 1
  out  <- x
  for (iter in seq_len(50L)) {
    out[is.na(x)] <- mu
    mu_new  <- mean(out)
    sig2_new <- stats::var(out)
    if (abs(mu_new - mu) < 1e-8) break
    mu <- mu_new; sig2 <- sig2_new
  }
  out
}

#' @keywords internal
.xmp_compare <- function(orig, imp, orig_name, imp_name, method) {
  line <- strrep("-", 72)
  message(line)
  message("  Imputation Results: method = ", method)
  message(line)
  nmiss  <- sum(is.na(orig))
  nfill  <- sum(!is.na(imp)) - sum(!is.na(orig))
  nstill <- sum(is.na(imp))
  message("  Missing before   : ", nmiss)
  message("  Values filled    : ", nfill)
  message("  Still missing    : ", nstill)

  o_mean <- mean(orig, na.rm = TRUE); i_mean <- mean(imp, na.rm = TRUE)
  o_sd   <- stats::sd(orig, na.rm = TRUE); i_sd <- stats::sd(imp, na.rm = TRUE)
  d_mean <- if (!is.na(o_mean) && o_mean != 0) (i_mean - o_mean) / abs(o_mean) * 100 else NA_real_

  message(sprintf("  %-12s %10s %10s %10s", "Stat", "Original", "Imputed", "Delta%"))
  message(sprintf("  %-12s %10.4f %10.4f %9.2f%%", "Mean", o_mean, i_mean, d_mean))
  message(sprintf("  %-12s %10.4f %10.4f", "SD", o_sd, i_sd))

  corr <- tryCatch(stats::cor(orig, imp, use = "complete.obs"), error = function(e) NA_real_)
  message(sprintf("  Correlation (obs pairs): %.4f", corr))
  message(line)

  list(n_miss = nmiss, n_filled = nfill, n_remain = nstill,
       orig_mean = o_mean, imp_mean = i_mean, d_mean_pct = d_mean,
       orig_sd = o_sd, imp_sd = i_sd, correlation = corr)
}

#' @keywords internal
.xmp_sensitivity <- function(data, target, pid, tid, knn_k) {
  methods <- c("mean", "median", "locf", "nocb", "linear", "spline",
               "pmm", "hotdeck", "knn", "rf", "em")
  orig      <- data[[target]]
  orig_mean <- mean(orig, na.rm = TRUE)

  line <- strrep("-", 72)
  message(line)
  message("  MODULE 4: Sensitivity Analysis -- Comparing Imputation Methods")
  message(line)
  message(sprintf("  %-12s %10s %10s %10s %10s %10s",
                  "Method", "Mean", "SD", "Min", "Max", "Delta Mean%"))
  message(line)

  results <- list()
  best_method <- methods[1L]; best_d <- Inf

  for (m in methods) {
    imp <- tryCatch(
      .xmp_impute(data, target, pid, tid, method = m, knn_k = knn_k),
      error = function(e) NULL
    )
    if (is.null(imp)) {
      message(sprintf("  %-12s  FAILED", m)); next
    }
    imean <- mean(imp, na.rm = TRUE); isd <- stats::sd(imp, na.rm = TRUE)
    imin  <- min(imp, na.rm = TRUE);  imax <- max(imp, na.rm = TRUE)
    d     <- if (!is.na(orig_mean) && orig_mean != 0)
      (imean - orig_mean) / abs(orig_mean) * 100 else 0
    message(sprintf("  %-12s %10.4f %10.4f %10.4f %10.4f %9.2f%%",
                    m, imean, isd, imin, imax, d))
    results[[m]] <- list(mean = imean, sd = isd, d_mean_pct = d)
    if (abs(d) < best_d) { best_d <- abs(d); best_method <- m }
  }
  message(line)
  message("  Best method (lowest distribution distortion): ", best_method)
  message("  Mean change: ", round(best_d, 2), "%")
  message(line)

  list(results = results, best_method = best_method, best_d_pct = best_d)
}

make_panel_na <- function(seed = 1) {
  set.seed(seed)
  df <- data.frame(
    id   = rep(1:4, each = 8),
    time = rep(1:8, times = 4),
    y    = rnorm(32),
    x1   = rnorm(32)
  )
  df$y[c(3, 11, 20, 27)] <- NA
  df
}

test_that("xtmispanel detect module runs without error", {
  df  <- make_panel_na()
  res <- xtmispanel(df, vars = c("y", "x1"), index = c("id", "time"), detect = TRUE)
  expect_type(res, "list")
  expect_named(res, "detect", ignore.order = TRUE)
  expect_equal(res$detect$total_missing, 4L)
})

test_that("xtmispanel imputation with mean works", {
  df  <- make_panel_na()
  res <- xtmispanel(df, vars = "y", index = c("id", "time"),
                    detect = FALSE, impute = "mean", target = "y")
  expect_false(is.null(res$imputed))
  expect_false(anyNA(res$imputed$y_imp))
})

test_that("xtmispanel imputation with locf works", {
  df  <- make_panel_na()
  res <- xtmispanel(df, vars = "y", index = c("id", "time"),
                    detect = FALSE, impute = "locf", target = "y")
  expect_named(res$imputed, c("id", "time", "y", "x1", "y_imp"), ignore.order = TRUE)
})

test_that("xtmispanel imputation with linear interpolation works", {
  df  <- make_panel_na()
  res <- xtmispanel(df, vars = "y", index = c("id", "time"),
                    detect = FALSE, impute = "linear", target = "y")
  expect_false(anyNA(res$imputed$y_imp))
})

test_that("xtmispanel impute_stats reports correlation", {
  df  <- make_panel_na()
  res <- xtmispanel(df, vars = "y", index = c("id", "time"),
                    detect = FALSE, impute = "mean", target = "y")
  expect_true(!is.null(res$impute_stats$correlation))
})

test_that("xtmispanel errors on missing index columns", {
  df <- make_panel_na()
  expect_error(xtmispanel(df, vars = "y", index = c("bad", "time")))
})

test_that("xtmispanel sensitivity runs and returns best_method", {
  df  <- make_panel_na()
  res <- xtmispanel(df, vars = "y", index = c("id", "time"),
                    detect = FALSE, sensitivity = TRUE, target = "y")
  expect_true(!is.null(res$sensitivity$best_method))
  expect_true(res$sensitivity$best_method %in%
                c("mean","median","locf","nocb","linear","spline",
                  "pmm","hotdeck","knn","rf","em"))
})

test_that(".xmp_locf fills forward correctly", {
  x   <- c(1, NA, NA, 4)
  res <- xtmispanel:::.xmp_locf(x)
  expect_equal(res, c(1, 1, 1, 4))
})

#' Tests of No Cross-Sectional Dependence in Panel Quantile Regressions
#'
#' Tests the null hypothesis of no cross-sectional error dependence (CSD) in
#' panel quantile regressions. Implements the T_tau and T-tilde_tau statistics
#' of Demetrescu, Hosseinkouchack and Rodrigues (2023).
#'
#' @param formula A formula of the form \code{y ~ x1 + x2 + ...}. Required for
#'   \code{mode = "pooled"} (default) and \code{mode = "individual"}.
#'   Not used when \code{residuals} is provided.
#' @param data A data frame containing the panel data in long format. Required
#'   unless \code{residuals} is provided.
#' @param index A character vector of length 2: \code{c("id_var", "time_var")}.
#'   Required unless \code{residuals} is provided.
#' @param quantiles A numeric vector of quantile levels, each strictly between
#'   0 and 1.
#' @param mode Estimation mode: \code{"pooled"} (default, pooled FE-QR),
#'   \code{"individual"} (per-unit QR), or \code{"residuals"} (provide
#'   pre-computed residuals via the \code{residuals} argument).
#' @param residuals A list (or named list) of numeric vectors or a matrix with
#'   one column per quantile, containing pre-computed QR residuals. Only used
#'   when \code{mode = "residuals"}.
#' @param bandwidth Numeric. KDE bandwidth for sparsity estimation. If
#'   \code{NULL} (default), uses \eqn{0.35 (NT)^{-0.2}}.
#' @param correction Logical. If \code{TRUE} (default), reports the
#'   bias-corrected T-tilde statistic in addition to T_tau.
#'
#' @return An object of class \code{"xtcsdq"} with components:
#'   \describe{
#'     \item{T_tau}{Numeric vector of T_tau statistics (one per quantile).}
#'     \item{Ttilde_tau}{Numeric vector of bias-corrected T-tilde_tau statistics.}
#'     \item{pval_T}{p-values for T_tau.}
#'     \item{pval_Ttilde}{p-values for T-tilde_tau.}
#'     \item{fhat}{KDE density estimates at zero (one per quantile).}
#'     \item{M_K}{Portmanteau statistic (average of T_tau over quantiles).}
#'     \item{Mtilde_K}{Bias-corrected portmanteau statistic.}
#'     \item{pval_M}{p-value for M_K.}
#'     \item{pval_Mc}{p-value for Mtilde_K.}
#'     \item{quantiles}{Quantile levels used.}
#'     \item{N}{Number of cross-sectional units.}
#'     \item{TT}{Number of time periods.}
#'     \item{bandwidth}{KDE bandwidth used.}
#'   }
#'
#' @details
#' The T_tau statistic (Equation 3 in Demetrescu et al., 2023) tests for CSD
#' by examining pairwise correlations of demeaned QR residuals across units.
#' Under the null of no CSD, T_tau is asymptotically standard normal.
#'
#' The bias-corrected version T-tilde_tau (Equation 5) subtracts two correction
#' terms that account for the estimation uncertainty in the QR slope and the
#' sparsity at the quantile. Reject H0 for large positive values.
#'
#' The portmanteau statistic \eqn{M_K = K^{-1} \sum_{q=1}^K T_\tau^{(q)}}
#' aggregates across K quantile levels.
#'
#' The KDE bandwidth defaults to \eqn{h = 0.35 (NT)^{-0.2}} as in the
#' original paper.
#'
#' @references
#' Demetrescu, M., Hosseinkouchack, M. and Rodrigues, P.M.M. (2023).
#' Testing for No Cross-Sectional Error Dependence in Panel Quantile
#' Regressions.
#' \emph{Ruhr Economic Papers}, No. 1041.
#' \doi{10.4419/96973002}
#'
#' @examples
#' \donttest{
#' set.seed(42)
#' n <- 8; tt <- 20
#' dat <- data.frame(
#'   id   = rep(1:n, each = tt),
#'   time = rep(1:tt, times = n),
#'   y    = rnorm(n * tt),
#'   x1   = rnorm(n * tt)
#' )
#' res <- xtcsdq(y ~ x1, data = dat, index = c("id", "time"),
#'               quantiles = c(0.25, 0.5, 0.75))
#' print(res)
#' summary(res)
#' }
#'
#' @export
xtcsdq <- function(formula = NULL, data = NULL, index = NULL,
                   quantiles,
                   mode = c("pooled", "individual", "residuals"),
                   residuals = NULL,
                   bandwidth = NULL,
                   correction = TRUE) {

  mode <- match.arg(mode)

  ## ── Validate inputs ──────────────────────────────────────────────────────
  if (!is.numeric(quantiles) || any(quantiles <= 0) || any(quantiles >= 1)) {
    stop("'quantiles' must be numeric with all values strictly between 0 and 1.",
         call. = FALSE)
  }
  quantiles <- sort(unique(quantiles))
  K         <- length(quantiles)

  if (mode == "residuals") {
    if (is.null(residuals)) {
      stop("'residuals' must be provided when mode = 'residuals'.", call. = FALSE)
    }
    # Normalize residuals to a matrix: T x K
    if (is.data.frame(residuals) || is.matrix(residuals)) {
      resid_mat <- as.matrix(residuals)
    } else if (is.list(residuals)) {
      resid_mat <- do.call(cbind, residuals)
    } else if (is.numeric(residuals) && K == 1L) {
      resid_mat <- matrix(residuals, ncol = 1L)
    } else {
      stop("'residuals' must be a matrix, data frame, or list of vectors.",
           call. = FALSE)
    }
    if (ncol(resid_mat) != K) {
      stop(sprintf("Number of columns in 'residuals' (%d) must equal length of 'quantiles' (%d).",
                   ncol(resid_mat), K), call. = FALSE)
    }
    # For residuals mode we need N and T; require index + data OR infer from dim
    if (is.null(data) || is.null(index)) {
      stop("'data' and 'index' are required even in 'residuals' mode to determine N and T.",
           call. = FALSE)
    }
  } else {
    if (is.null(formula) || is.null(data) || is.null(index)) {
      stop("'formula', 'data', and 'index' are required for mode = '", mode, "'.",
           call. = FALSE)
    }
    if (!inherits(formula, "formula")) {
      stop("'formula' must be a formula object.", call. = FALSE)
    }
    if (!is.character(index) || length(index) != 2) {
      stop("'index' must be a character vector of length 2.", call. = FALSE)
    }
    if (!all(index %in% names(data))) {
      stop("Variables in 'index' not found in 'data'.", call. = FALSE)
    }
  }

  ## ── Panel structure ───────────────────────────────────────────────────────
  ivar <- index[1]
  tvar <- index[2]

  panels <- sort(unique(data[[ivar]]))
  N      <- length(panels)
  times  <- sort(unique(data[[tvar]]))
  TT     <- length(times)

  if (N < 2) stop("At least 2 cross-sectional units are required.", call. = FALSE)
  if (TT < 3) stop("At least 3 time periods are required.", call. = FALSE)

  ## Check balance
  obs_count <- as.integer(table(data[[ivar]]))
  if (length(unique(obs_count)) > 1) {
    stop("Panel must be balanced (all units must have the same number of observations).",
         call. = FALSE)
  }

  ## ── KDE bandwidth ─────────────────────────────────────────────────────────
  if (is.null(bandwidth) || bandwidth <= 0) {
    bw_used <- 0.35 * (N * TT)^(-0.2)
  } else {
    bw_used <- bandwidth
  }

  ## ── Get/arrange residuals ─────────────────────────────────────────────────
  # resid_list[[q]] is an N x TT matrix of demeaned residuals for quantile q
  resid_list <- vector("list", K)

  if (mode == "residuals") {
    ## Residuals provided: reshape to N x TT per quantile
    for (qi in seq_len(K)) {
      r_vec <- resid_mat[, qi]
      Rmat  <- .reshape_resid(r_vec, data, ivar, tvar, panels, N, TT)
      resid_list[[qi]] <- Rmat
    }
  } else if (mode == "individual") {
    mf_vars <- stats::model.frame(formula, data = data, na.action = stats::na.omit)
    depv    <- names(mf_vars)[1]
    indv    <- names(mf_vars)[-1]
    for (qi in seq_len(K)) {
      raw_resid <- rep(NA_real_, nrow(data))
      for (pi in seq_along(panels)) {
        idx      <- which(data[[ivar]] == panels[pi])
        sub_data <- data[idx, , drop = FALSE]
        sub_mf   <- tryCatch(
          stats::model.frame(formula, data = sub_data, na.action = stats::na.omit),
          error = function(e) NULL
        )
        if (is.null(sub_mf) || nrow(sub_mf) < length(indv) + 2) next
        Y_i <- sub_mf[[1]]
        X_i <- as.matrix(sub_mf[, -1, drop = FALSE])
        fit <- tryCatch(.qr_irls(Y_i, cbind(1, X_i), quantiles[qi]),
                        error = function(e) NULL)
        if (is.null(fit)) next
        raw_resid[idx] <- Y_i - cbind(1, X_i) %*% fit
      }
      Rmat <- .reshape_resid(raw_resid, data, ivar, tvar, panels, N, TT)
      resid_list[[qi]] <- Rmat
    }
  } else {
    ## pooled FE-QR
    mf_vars <- stats::model.frame(formula, data = data, na.action = stats::na.omit)
    depv    <- names(mf_vars)[1]
    indv    <- names(mf_vars)[-1]
    data_s  <- data[order(data[[ivar]], data[[tvar]]), , drop = FALSE]
    # Create dummy FE dummies (within-demeaning approach: subtract unit means)
    for (qi in seq_len(K)) {
      Y_vec <- mf_vars[[1]]
      X_mat <- as.matrix(mf_vars[, -1, drop = FALSE])
      # Within transformation
      uid  <- data[[ivar]]
      Yw   <- Y_vec - tapply(Y_vec, uid, mean)[as.character(uid)]
      group_means <- do.call(rbind, lapply(split(seq_len(nrow(X_mat)), uid), function(i) { matrix(colMeans(X_mat[i, , drop = FALSE]), 1) }))
  rownames(group_means) <- names(split(seq_len(nrow(X_mat)), uid))
  Xw   <- X_mat - group_means[as.character(uid), , drop = FALSE]

      fit  <- tryCatch(.qr_irls(as.numeric(Yw), cbind(1, Xw), quantiles[qi]),
                       error = function(e) NULL)
      if (is.null(fit)) {
        raw_resid <- rep(NA_real_, nrow(data))
      } else {
        raw_resid <- as.numeric(Yw) - cbind(1, Xw) %*% fit
      }
      Rmat <- .reshape_resid(raw_resid, data, ivar, tvar, panels, N, TT)
      resid_list[[qi]] <- Rmat
    }
  }

  ## ── Compute test statistics per quantile ──────────────────────────────────
  T_tau_vec    <- numeric(K)
  Ttilde_vec   <- numeric(K)
  pval_T_vec   <- numeric(K)
  pval_Tt_vec  <- numeric(K)
  fhat_vec     <- numeric(K)

  for (qi in seq_len(K)) {
    Rmat <- resid_list[[qi]]
    if (is.null(Rmat) || any(is.na(Rmat))) {
      T_tau_vec[qi]   <- NA_real_
      Ttilde_vec[qi]  <- NA_real_
      pval_T_vec[qi]  <- NA_real_
      pval_Tt_vec[qi] <- NA_real_
      fhat_vec[qi]    <- NA_real_
      next
    }

    ## Demean: subtract unit means
    unit_means <- rowMeans(Rmat)
    Rmd        <- Rmat - unit_means  # N x TT demeaned

    ## Standardize by unit SD for KDE
    unit_sd <- apply(Rmd, 1, stats::sd)
    unit_sd[unit_sd < 1e-14] <- 1e-14
    Rstd    <- Rmat / unit_sd   # N x TT (raw / sigma_i)

    ## KDE estimate of density at zero
    all_std <- as.numeric(Rstd)
    h       <- bw_used
    fhat    <- mean(stats::dnorm(all_std / h) / h)
    fhat_vec[qi] <- fhat

    ## Compute T_tau via pairwise correlations on demeaned residuals
    T_tau_val <- .compute_T_tau(Rmd, N, TT)

    ## Bias corrections
    corr1 <- sqrt(N * (N - 1) / (2 * TT))
    tau_q  <- quantiles[qi]
    corr2  <- (tau_q * (1 - tau_q)) / max(fhat^2, 1e-14) *
              sqrt(N * (N - 1) / TT)
    Ttilde_val <- T_tau_val - corr1 - corr2

    T_tau_vec[qi]   <- T_tau_val
    Ttilde_vec[qi]  <- Ttilde_val
    pval_T_vec[qi]  <- 1 - stats::pnorm(T_tau_val)
    pval_Tt_vec[qi] <- 1 - stats::pnorm(Ttilde_val)
  }

  ## ── Portmanteau M_K ───────────────────────────────────────────────────────
  M_K      <- mean(T_tau_vec, na.rm = TRUE)
  Mtilde_K <- mean(Ttilde_vec, na.rm = TRUE)
  pval_M   <- 1 - stats::pnorm(M_K)
  pval_Mc  <- 1 - stats::pnorm(Mtilde_K)

  ## ── Output ────────────────────────────────────────────────────────────────
  out <- list(
    T_tau      = T_tau_vec,
    Ttilde_tau = Ttilde_vec,
    pval_T     = pval_T_vec,
    pval_Ttilde = pval_Tt_vec,
    fhat       = fhat_vec,
    M_K        = M_K,
    Mtilde_K   = Mtilde_K,
    pval_M     = pval_M,
    pval_Mc    = pval_Mc,
    quantiles  = quantiles,
    N          = N,
    TT         = TT,
    bandwidth  = bw_used,
    correction = correction,
    mode       = mode
  )
  class(out) <- "xtcsdq"
  out
}


## ── Internal helpers ─────────────────────────────────────────────────────

#' @keywords internal
.reshape_resid <- function(r_vec, data, ivar, tvar, panels, N, TT) {
  Rmat <- matrix(NA_real_, nrow = N, ncol = TT)
  times <- sort(unique(data[[tvar]]))
  for (pi in seq_along(panels)) {
    idx <- which(data[[ivar]] == panels[pi])
    sub <- data[idx, c(ivar, tvar), drop = FALSE]
    sub <- sub[order(sub[[tvar]]), , drop = FALSE]
    ti  <- match(sub[[tvar]], times)
    Rmat[pi, ti] <- r_vec[idx[order(data[idx, tvar])]]
  }
  Rmat
}

#' @keywords internal
.compute_T_tau <- function(Rmd, N, TT) {
  ## T_tau = sum_{i<j} (T * rho_ij^2 - 1) / sqrt(N(N-1))
  ## rho_ij = cor of demeaned resids i and j
  rho2_sum <- 0
  for (i in seq_len(N - 1)) {
    u_i  <- Rmd[i, ]
    ss_i <- sum(u_i^2)
    for (j in (i + 1):N) {
      u_j    <- Rmd[j, ]
      ss_j   <- sum(u_j^2)
      cov_ij <- sum(u_i * u_j)
      denom  <- ss_i * ss_j
      if (denom < 1e-30) next
      rho_sq   <- cov_ij^2 / denom
      rho2_sum <- rho2_sum + (TT * rho_sq - 1)
    }
  }
  rho2_sum / sqrt(N * (N - 1))
}

#' @keywords internal
.qr_irls <- function(y, X, tau, maxit = 200L, tol = 1e-8) {
  n    <- length(y)
  beta <- stats::lm.fit(X, y)$coefficients
  for (iter in seq_len(maxit)) {
    resid    <- y - X %*% beta
    w        <- pmax(abs(resid), tol)
    w        <- ifelse(resid >= 0, tau / w, (1 - tau) / w)
    W        <- diag(as.numeric(w), n)
    XtWX     <- crossprod(X, W %*% X)
    XtWy     <- crossprod(X, W %*% y)
    beta_new <- tryCatch(solve(XtWX, XtWy), error = function(e) beta)
    if (max(abs(beta_new - beta)) < tol) { beta <- beta_new; break }
    beta <- beta_new
  }
  as.numeric(beta)
}


## ── S3 methods ───────────────────────────────────────────────────────────

#' Print method for xtcsdq objects
#'
#' @param x An object of class \code{"xtcsdq"}.
#' @param ... Additional arguments (ignored).
#' @return Invisibly returns \code{x}.
#' @export
print.xtcsdq <- function(x, ...) {
  cat("\n")
  cat(strrep("-", 70), "\n")
  cat("  XTCSDQ: CSD Test in Panel Quantile Regressions\n")
  cat("  Demetrescu, Hosseinkouchack & Rodrigues (2023)\n")
  cat(strrep("-", 70), "\n")
  cat(sprintf("  N (units)  : %d\n", x$N))
  cat(sprintf("  T (periods): %d\n", x$TT))
  cat(sprintf("  Quantiles  : %s\n", paste(x$quantiles, collapse = ", ")))
  cat(sprintf("  Bandwidth  : %.6f\n", x$bandwidth))
  cat(sprintf("  Mode       : %s\n", x$mode))
  cat(strrep("-", 70), "\n")
  cat("  H0: No cross-sectional error dependence in panel QR\n")
  cat("  (Reject for large positive values)\n")
  cat(strrep("-", 70), "\n")

  if (x$correction) {
    cat(sprintf("  %-8s  %10s  %10s  %10s  %10s\n",
                "tau", "T_tau", "T~_tau", "p(T)", "p(T~)"))
  } else {
    cat(sprintf("  %-8s  %10s  %10s\n", "tau", "T_tau", "p(T)"))
  }
  cat(strrep("-", 70), "\n")

  for (i in seq_along(x$quantiles)) {
    if (x$correction) {
      cat(sprintf("  %-8.3f  %10.4f  %10.4f  %10.4f  %10.4f  %s\n",
                  x$quantiles[i], x$T_tau[i], x$Ttilde_tau[i],
                  x$pval_T[i], x$pval_Ttilde[i],
                  .csd_stars(x$pval_Ttilde[i])))
    } else {
      cat(sprintf("  %-8.3f  %10.4f  %10.4f  %s\n",
                  x$quantiles[i], x$T_tau[i], x$pval_T[i],
                  .csd_stars(x$pval_T[i])))
    }
  }

  if (length(x$quantiles) > 1) {
    cat(strrep("-", 70), "\n")
    if (x$correction) {
      cat(sprintf("  %-8s  %10.4f  %10.4f  %10.4f  %10.4f  %s\n",
                  "M_K", x$M_K, x$Mtilde_K, x$pval_M, x$pval_Mc,
                  .csd_stars(x$pval_Mc)))
    } else {
      cat(sprintf("  %-8s  %10.4f  %10.4f  %s\n",
                  "M_K", x$M_K, x$pval_M, .csd_stars(x$pval_M)))
    }
  }
  cat(strrep("-", 70), "\n")
  cat("  *** p<0.01, ** p<0.05, * p<0.10\n\n")

  invisible(x)
}

#' Summary method for xtcsdq objects
#'
#' @param object An object of class \code{"xtcsdq"}.
#' @param ... Additional arguments (ignored).
#' @return Invisibly returns \code{object}.
#' @export
summary.xtcsdq <- function(object, ...) {
  print(object, ...)
  cat("  KDE density estimates at zero (fhat):\n")
  for (i in seq_along(object$quantiles)) {
    cat(sprintf("    tau=%.2f: fhat=%.6f\n", object$quantiles[i], object$fhat[i]))
  }
  cat("\n")
  invisible(object)
}

#' @keywords internal
.csd_stars <- function(p) {
  if (is.na(p)) return("")
  if (p < 0.01) return("***")
  if (p < 0.05) return("**")
  if (p < 0.10) return("*")
  return("")
}

test_that("xtcsdq returns correct structure", {
  set.seed(123)
  n <- 8; tt <- 20
  dat <- data.frame(
    id   = rep(1:n, each = tt),
    time = rep(1:tt, times = n),
    y    = rnorm(n * tt),
    x1   = rnorm(n * tt)
  )
  res <- xtcsdq(y ~ x1, data = dat, index = c("id", "time"),
                quantiles = c(0.25, 0.5, 0.75))
  expect_s3_class(res, "xtcsdq")
  expect_equal(res$N, n)
  expect_equal(res$TT, tt)
  expect_length(res$T_tau, 3)
  expect_length(res$pval_T, 3)
})

test_that("xtcsdq portmanteau stat is mean of T_tau", {
  set.seed(42)
  n <- 6; tt <- 15
  dat <- data.frame(
    id   = rep(1:n, each = tt),
    time = rep(1:tt, times = n),
    y    = rnorm(n * tt),
    x1   = rnorm(n * tt)
  )
  res <- xtcsdq(y ~ x1, data = dat, index = c("id", "time"),
                quantiles = c(0.25, 0.75))
  expect_equal(res$M_K, mean(res$T_tau), tolerance = 1e-10)
})

test_that("xtcsdq p-values are in [0,1]", {
  set.seed(7)
  n <- 5; tt <- 12
  dat <- data.frame(
    id   = rep(1:n, each = tt),
    time = rep(1:tt, times = n),
    y    = rnorm(n * tt),
    x1   = rnorm(n * tt)
  )
  res <- xtcsdq(y ~ x1, data = dat, index = c("id", "time"),
                quantiles = 0.5)
  expect_true(all(res$pval_T >= 0 & res$pval_T <= 1))
  expect_true(all(res$pval_Ttilde >= 0 & res$pval_Ttilde <= 1))
})

test_that("print and summary do not error", {
  set.seed(1)
  n <- 5; tt <- 12
  dat <- data.frame(
    id   = rep(1:n, each = tt),
    time = rep(1:tt, times = n),
    y    = rnorm(n * tt),
    x1   = rnorm(n * tt)
  )
  res <- xtcsdq(y ~ x1, data = dat, index = c("id", "time"),
                quantiles = c(0.25, 0.5))
  expect_output(print(res))
  expect_output(summary(res))
})

\name{xtpretest}
\alias{xtpretest}
\title{Comprehensive Panel Data Pre-Testing Suite}
\usage{
xtpretest(
  data,
  formula,
  index,
  tests = "ALL",
  level = 0.05
)
}
\arguments{
\item{data}{A \code{data.frame} in long format containing all variables.}

\item{formula}{A two-sided formula of the form \code{y ~ x1 + x2 + ...}
specifying the dependent and independent variables.}

\item{index}{Character vector of length 2: \code{c("panel_id", "time_id")}.}

\item{tests}{Character vector. Which modules to run. Possible values:
\code{"summary"}, \code{"hsiao"}, \code{"robust"}, \code{"heterogeneity"},
\code{"csd"}.  Default \code{"ALL"} runs everything.}

\item{level}{Numeric. Significance level for decisions (default 0.05).}
}
\value{
A list (invisibly) with components:
\describe{
  \item{\code{summary}}{Panel summary statistics.}
  \item{\code{hsiao}}{Hsiao homogeneity F-test results.}
  \item{\code{robust}}{Robust HC1 F-test results.}
  \item{\code{swamy}}{Swamy heterogeneity test results.}
  \item{\code{csd}}{Cross-sectional dependence test results.}
  \item{\code{recommendation}}{Character. Suggested estimator.}
}
}
\description{
Performs a full battery of panel data pre-tests: Hsiao (2014) homogeneity
F-tests, robust (HC1) versions, Swamy (1970) parameter heterogeneity test,
cross-sectional dependence (Pesaran 2004), and panel summary statistics.
}
\examples{
set.seed(10)
n <- 5; t <- 10
df <- data.frame(
  id   = rep(1:n, each = t),
  time = rep(1:t, times = n),
  y    = rnorm(n * t),
  x1   = rnorm(n * t)
)
res <- xtpretest(df, y ~ x1, index = c("id", "time"),
                 tests = c("hsiao", "csd"))
}
\references{
Hsiao, C. (2014). \emph{Analysis of Panel Data} (3rd ed.).
Cambridge University Press.
\doi{10.1017/CBO9781139839327}

Swamy, P. A. V. B. (1970). Efficient inference in a random coefficient
regression model. \emph{Econometrica}, 38(2), 311-323.
\doi{10.2307/1909405}

Pesaran, M. H. (2004). General diagnostic tests for cross section
dependence in panels. Cambridge Working Paper in Economics, No. 0435.
\doi{10.2139/ssrn.572504}
}

\name{print.xtqsh}
\alias{print.xtqsh}
\title{Print Method for xtqsh Objects}
\description{Prints a formatted summary of an \code{"xtqsh"} test result.}
\usage{
\method{print}{xtqsh}(x, \dots)
}
\arguments{
  \item{x}{An object of class \code{"xtqsh"}.}
  \item{\dots}{Additional arguments (ignored).}
}
\value{Invisibly returns \code{x}.}
\keyword{print}

\name{xtqsh}
\alias{xtqsh}
\title{Quantile Regression Slope Homogeneity Test for Panel Data}
\description{
  Tests the null hypothesis of slope homogeneity in panel quantile regressions.
  Implements the S-hat and D-hat statistics of Galvao et al. (2017).
}
\usage{
xtqsh(formula, data, index, tau, bw = "hallsheather", marginal = FALSE)
}
\arguments{
  \item{formula}{A formula of the form \code{y ~ x1 + x2 + ...}.}
  \item{data}{A data frame containing the panel data in long format.}
  \item{index}{Character vector of length 2: \code{c("id_var", "time_var")}.}
  \item{tau}{Numeric vector of quantile levels, each strictly between 0 and 1.}
  \item{bw}{Bandwidth method: \code{"hallsheather"} (default) or \code{"bofinger"}.}
  \item{marginal}{Logical. If \code{TRUE}, compute per-variable marginal tests. Default \code{FALSE}.}
}
\value{
  An object of class \code{"xtqsh"} containing test statistics and p-values.
}
\references{
Galvao, A.F., Juhl, T., Montes-Rojas, G. and Olmo, J. (2017).
Testing Slope Homogeneity in Quantile Regression Panel Data.
\emph{Journal of Financial Econometrics}, 16(2), 211-243.
}
\examples{
\donttest{
set.seed(42)
n <- 10; tt <- 20
dat <- data.frame(
  id   = rep(1:n, each = tt),
  time = rep(1:tt, times = n),
  y    = rnorm(n * tt),
  x1   = rnorm(n * tt)
)
res <- xtqsh(y ~ x1, data = dat, index = c("id", "time"), tau = 0.5)
print(res)
}
}
\keyword{htest}

\name{xtmispanel}
\alias{xtmispanel}
\title{Missing Data Detection and Imputation for Panel Data}
\usage{
xtmispanel(
  data,
  vars = NULL,
  index,
  detect = TRUE,
  test = FALSE,
  impute = NULL,
  target = NULL,
  new_var = NULL,
  sensitivity = FALSE,
  knn_k = 5L
)
}
\arguments{
\item{data}{A \code{data.frame} in long format.}

\item{vars}{Character vector of variable names to analyse.  If \code{NULL}
(default), all numeric columns except the index are used.}

\item{index}{Character vector of length 2: \code{c("panel_id", "time_id")}.}

\item{detect}{Logical. Run Module 1 (detection tables, default \code{TRUE}).}

\item{test}{Logical. Run Module 2 (MCAR/MAR mechanism tests,
default \code{FALSE}).}

\item{impute}{Character or \code{NULL}. If a method name is given, run
Module 3 (imputation).  Supported methods:
\code{"mean"}, \code{"median"}, \code{"locf"}, \code{"nocb"},
\code{"linear"}, \code{"spline"}, \code{"pmm"}, \code{"hotdeck"},
\code{"knn"}, \code{"rf"}, \code{"em"}.}

\item{target}{Character. Name of the variable to impute (required when
\code{impute} is not \code{NULL}).}

\item{new_var}{Character. Name of the output imputed variable
(default \code{"\{target\}_imp"}).}

\item{sensitivity}{Logical. Run Module 4 (sensitivity analysis across
all imputation methods, default \code{FALSE}).}

\item{knn_k}{Integer. Number of neighbours for KNN imputation (default 5).}
}
\value{
A list (invisibly) with components:
\describe{
  \item{\code{detect}}{Summary statistics per variable/panel/period.}
  \item{\code{test}}{MCAR and MAR test results.}
  \item{\code{imputed}}{The \code{data} frame augmented with the imputed
    column (when imputation is requested).}
  \item{\code{impute_stats}}{Summary comparing original vs imputed.}
  \item{\code{sensitivity}}{Sensitivity analysis results.}
}
}
\description{
Detects, diagnoses, and imputes missing values in panel (longitudinal)
data sets.  The function can produce summary tables (Module 1), test the
missingness mechanism (Module 2), impute a target variable (Module 3),
and run a cross-method sensitivity analysis (Module 4).
}
\examples{
set.seed(1)
df <- data.frame(
  id   = rep(1:4, each = 8),
  time = rep(1:8, times = 4),
  y    = c(rnorm(32))
)
# introduce some NAs
df$y[c(3, 11, 20)] <- NA
res <- xtmispanel(df, vars = "y", index = c("id", "time"), detect = TRUE)
}
\references{
Little, R. J. A. (1988). A test of missing completely at random for
multivariate data with missing values.
\emph{Journal of the American Statistical Association}, 83(404), 1198-1202.
\doi{10.1080/01621459.1988.10478714}
}

\name{qsh_sample}
\alias{qsh_sample}
\docType{data}
\title{Sample Panel Dataset for Quantile Slope Homogeneity Testing}
\description{
A simulated balanced panel dataset for demonstrating the quantile slope
homogeneity test (\code{\link{xtqsh}}).
}
\usage{data(qsh_sample)}
\format{
A data frame with columns:
\describe{
  \item{id}{Cross-sectional unit identifier.}
  \item{time}{Time period identifier.}
  \item{y}{Dependent variable.}
  \item{x1}{First explanatory variable.}
  \item{x2}{Second explanatory variable.}
}
}
\keyword{datasets}

\name{summary.xtqsh}
\alias{summary.xtqsh}
\title{Summary Method for xtqsh Objects}
\description{Prints a summary of an \code{"xtqsh"} test result.}
\usage{
\method{summary}{xtqsh}(object, \dots)
}
\arguments{
  \item{object}{An object of class \code{"xtqsh"}.}
  \item{\dots}{Additional arguments (ignored).}
}
\value{Invisibly returns \code{object}.}
\keyword{print}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xtcsdq.R
\name{xtcsdq}
\alias{xtcsdq}
\title{Tests of No Cross-Sectional Dependence in Panel Quantile Regressions}
\usage{
xtcsdq(
  formula = NULL,
  data = NULL,
  index = NULL,
  quantiles,
  mode = c("pooled", "individual", "residuals"),
  residuals = NULL,
  bandwidth = NULL,
  correction = TRUE
)
}
\arguments{
\item{formula}{A formula of the form \code{y ~ x1 + x2 + ...}. Required for
\code{mode = "pooled"} (default) and \code{mode = "individual"}.
Not used when \code{residuals} is provided.}

\item{data}{A data frame containing the panel data in long format. Required
unless \code{residuals} is provided.}

\item{index}{A character vector of length 2: \code{c("id_var", "time_var")}.
Required unless \code{residuals} is provided.}

\item{quantiles}{A numeric vector of quantile levels, each strictly between
0 and 1.}

\item{mode}{Estimation mode: \code{"pooled"} (default, pooled FE-QR),
\code{"individual"} (per-unit QR), or \code{"residuals"} (provide
pre-computed residuals via the \code{residuals} argument).}

\item{residuals}{A list (or named list) of numeric vectors or a matrix with
one column per quantile, containing pre-computed QR residuals. Only used
when \code{mode = "residuals"}.}

\item{bandwidth}{Numeric. KDE bandwidth for sparsity estimation. If
\code{NULL} (default), uses \eqn{0.35 (NT)^{-0.2}}.}

\item{correction}{Logical. If \code{TRUE} (default), reports the
bias-corrected T-tilde statistic in addition to T_tau.}
}
\value{
An object of class \code{"xtcsdq"} with components:
  \describe{
    \item{T_tau}{Numeric vector of T_tau statistics (one per quantile).}
    \item{Ttilde_tau}{Numeric vector of bias-corrected T-tilde_tau statistics.}
    \item{pval_T}{p-values for T_tau.}
    \item{pval_Ttilde}{p-values for T-tilde_tau.}
    \item{fhat}{KDE density estimates at zero (one per quantile).}
    \item{M_K}{Portmanteau statistic (average of T_tau over quantiles).}
    \item{Mtilde_K}{Bias-corrected portmanteau statistic.}
    \item{pval_M}{p-value for M_K.}
    \item{pval_Mc}{p-value for Mtilde_K.}
    \item{quantiles}{Quantile levels used.}
    \item{N}{Number of cross-sectional units.}
    \item{TT}{Number of time periods.}
    \item{bandwidth}{KDE bandwidth used.}
  }
}
\description{
Tests the null hypothesis of no cross-sectional error dependence (CSD) in
panel quantile regressions. Implements the T_tau and T-tilde_tau statistics
of Demetrescu, Hosseinkouchack and Rodrigues (2023).
}
\details{
The T_tau statistic (Equation 3 in Demetrescu et al., 2023) tests for CSD
by examining pairwise correlations of demeaned QR residuals across units.
Under the null of no CSD, T_tau is asymptotically standard normal.

The bias-corrected version T-tilde_tau (Equation 5) subtracts two correction
terms that account for the estimation uncertainty in the QR slope and the
sparsity at the quantile. Reject H0 for large positive values.

The portmanteau statistic \eqn{M_K = K^{-1} \sum_{q=1}^K T_\tau^{(q)}}
aggregates across K quantile levels.

The KDE bandwidth defaults to \eqn{h = 0.35 (NT)^{-0.2}} as in the
original paper.
}
\examples{
\donttest{
set.seed(42)
n <- 8; tt <- 20
dat <- data.frame(
  id   = rep(1:n, each = tt),
  time = rep(1:tt, times = n),
  y    = rnorm(n * tt),
  x1   = rnorm(n * tt)
)
res <- xtcsdq(y ~ x1, data = dat, index = c("id", "time"),
              quantiles = c(0.25, 0.5, 0.75))
print(res)
summary(res)
}

}
\references{
Demetrescu, M., Hosseinkouchack, M. and Rodrigues, P.M.M. (2023).
Testing for No Cross-Sectional Error Dependence in Panel Quantile
Regressions.
\emph{Ruhr Economic Papers}, No. 1041.
\doi{10.4419/96973002}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xtcsdq.R
\name{summary.xtcsdq}
\alias{summary.xtcsdq}
\title{Summary method for xtcsdq objects}
\usage{
\method{summary}{xtcsdq}(object, ...)
}
\arguments{
\item{object}{An object of class \code{"xtcsdq"}.}

\item{...}{Additional arguments (ignored).}
}
\value{
Invisibly returns \code{object}.
}
\description{
Summary method for xtcsdq objects
}
